#!/usr/bin/env python
#
# fio-attach.py		Emulate Fusion-io fio-attach command-line utility
#

import sys
if sys.hexversion < 0x02060500:
    print >> sys.stderr, "This script requires python 2.6.5 or higher."
    exit(1)

try:
    import argparse
except:
    print >> sys.stderr, "Please install argparse from http://code.google.com/p/argparse."
    exit(1)

try:
    import pywbem
except ImportError:
    print >> sys.stderr, "Please install the pywbem module from http://sourceforge.net/projects/pywbem."
    exit(1)

import time
import string
import fio_claimlib

__author__ = "Fusion-io management software team"
__date__   = "$Feb 14, 2011 10:20:55 AM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

# define iodimm states

st_Unknown = 0
st_Detached = 1
st_Attaching = 2
st_Scanning = 3
st_Attached = 4
st_Detaching = 5
st_Stopping = 6
st_Stopped = 7
st_Formatting = 8
st_Minimal = 9
st_Failed = 10
st_Updating = 11

def make_wbem_url(args):
    return "{0}://{1}".format('https' if args.secure_flag else 'http', args.server)

def attach(conn, ioport_name):
    return conn.InvokeMethod('Attach', ioport_name)

def iomp_state(conn, iname):
    instance = conn.GetInstance(iname)
    return instance.properties['State'].value

def iomp_state_to_string(conn, state):
    iclass = conn.GetClass('FIO_IoMemoryPort')
    return iclass.properties['State'].qualifiers['Values'].value[state]

# ---------------------------------------------------------------------------
# START OF MAIN

# Setup argparse data structures and parse command-line arguments

description = '''Fusion-io %(prog)s utility (3.1.4.111 tahoe@a2a30d5308e5M).
Copyright 2006-2011 Fusion-io, Inc. All rights reserved.'''

parser = argparse.ArgumentParser(prog='fio-attach', description=description, formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument('device', metavar='<device>', nargs=1, help='a pathname to the device (/dev/fctX)');
parser.add_argument('-v', '--version', action='version', version='%(prog)s version 3.1.4.111 tahoe@a2a30d5308e5M', help="print version and exit")
parser.add_argument('-q', '--quiet', action='store_true', dest='quiet_flag', help='don\'t display progress')
parser.add_argument('-s', '--skip-claim', action='store_true', dest='skip_claim', help='skip claim rules checks (ESX/ESXi only), may cause system instability if device is in use)')

wbemgroup = parser.add_argument_group('remote options')
wbemgroup.add_argument('--timeout', metavar='<seconds>', default='30', dest='timeout', type=int, help='seconds to wait for request to complete (default:30; a timeout value of 0 means don\'t wait)')
wbemgroup.add_argument('--server', metavar='<address>', dest='server', required=True, help='remote host address/dns name')
wbemgroup.add_argument('--namespace', metavar='<namespace>', dest='namespace', default='fio/cimv2', help='CIM namespace (default:fio/cimv2)')
wbemgroup.add_argument('--username', metavar='<username>', dest='username', default='anonymous', help='remote user (default:anonymous)')
wbemgroup.add_argument('--password', metavar='<password>', dest='password', default='', help="remote user's password (default:none)")
wbemgroup.add_argument('--no-ssl', default=True, action='store_false', dest='secure_flag', help='disable secure http on connection');

args = parser.parse_args()

# parse path name (/dev/fctx); extract fctx

short_device = string.split(args.device[0], '/')[-1]

# build a URL and credentials tuple from command line arguments

url = make_wbem_url(args)
creds = (args.username, args.password) if args.secure_flag else None

# establish connection from the URL

conn = pywbem.WBEMConnection(url, creds, default_namespace=args.namespace);

# enumerate all FIO_IoMemoryPort instances; look for target device

device_name = None
iomport_inames = conn.EnumerateInstanceNames('FIO_IoMemoryPort')
for iname in iomport_inames:
    if iname.keybindings['DeviceID'] == short_device:
        device_name = iname
        break

if device_name == None:
    print >> sys.stderr, "Error: Unable to locate device '%s'." % args.device[0]
    exit(1)

if iomp_state(conn, device_name) == st_Attached:
    print >> sys.stderr, "Error: device '%s' is already attached." % args.device[0]
    exit(1)

# send attach request

if not args.quiet_flag:
    print "Sending 'attach' request to %s for device %s." % (url, args.device[0])

# if targeting an ESX system, check for other unclaimed devices before attaching

if args.namespace == "fio/cimv2" and not args.skip_claim:
    if not args.quiet_flag:
        print "Checking for unclaimed(detached) devices..."
    unclaimed = fio_claimlib.unclaimed(args)
    if unclaimed is not None:
        print "Cannot attach '%s': Another device '%s' was found in the unclaimed(detached) " \
              "state and must be attached before proceeding" % (args.device[0], unclaimed)
        exit(1)

rv = attach(conn, iname)
if not rv[0]:
    print >> sys.stderr, "Attach request failed."
    exit(1)

# check status while attach is not complete and we haven't timed out

timeout = args.timeout
state = iomp_state(conn, device_name)
if not args.quiet_flag and args.timeout != 0:
    print "Attaching",
    sys.stdout.flush()
while timeout > 0 and not state == st_Attached:
    time.sleep(1)
    if not args.quiet_flag:
        print ".",
        sys.stdout.flush()
    state = iomp_state(conn, device_name)
    timeout = timeout - 1

# check for success, failure, or timeout

if not args.quiet_flag and args.timeout != 0:
    if state == st_Attached:
        print "complete."
    elif state == st_Failed:
        print "failed."
    else:
        print "request timed out."

# if targeting an ESX system, run the claim rules after attaching

if args.namespace == "fio/cimv2" and not args.skip_claim:
    if state == st_Attached and args.timeout != 0:
        if not args.quiet_flag:
            print "Claiming device after attach..."
        fio_claimlib.claim(args)
        if not args.quiet_flag:
            print "Done."

# display state string and error on failure and terminate with error

if state != st_Attached:
    if args.timeout != 0:
        print "Error: failed to attach device '%s' for unknown reason." % args.device[0]
        print "Last known state of '%s': %s (%d)." \
                % (args.device[0], iomp_state_to_string(conn, state), state)
    exit(1)

# END OF MAIN
# ---------------------------------------------------------------------------
