#!/usr/bin/env python
#
# fio-format.py		Emulate Fusion-io fio-format command-line utility
#

import sys
if sys.hexversion < 0x02060500:
    print >> sys.stderr, "This script requires python 2.6.5 or higher."
    exit(1)

try:
    import argparse
except ImportError:
    print >> sys.stderr, "Please install argparse from http://code.google.com/p/argparse."
    exit(1)

try:
    import pywbem
except ImportError:
    print >> sys.stderr, "Please install the pywbem module from http://sourceforge.net/projects/pywbem." 
    exit(1)

import time
import string

__author__ = "Fusion-io management software team"
__date__ ="$Feb 15, 2011 1:42:17 PM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

# Useful constants

one_mibyte = 1024 ** 2
one_gibyte = 1024 ** 3
one_mbyte  = 1000 ** 2
one_gbyte  = 1000 ** 3

# Format modes

mode_percent = 0
mode_mbytes = 1

# IoMemoryPort states

st_Unknown = 0
st_Detached = 1
st_Attaching = 2
st_Scanning = 3
st_Attached = 4
st_Detaching = 5
st_Stopping = 6
st_Stopped = 7
st_Formatting = 8
st_Minimal = 9
st_Failed = 10
st_Updating = 11
st_Busy = 12

# constants default defaults

DEFAULT_BLOCK_SIZE = 512
MIN_BLOCK_SIZE = 512
MAX_BLOCK_SIZE = 4096

# Subroutines

def make_wbem_url(args):
    """Build and return an http or https url based on args."""
    return "http%s://%s" % ('s' if args.secure_flag else '', args.server)

def parse_size(size, min_bytes, mfactor):
    """Parse a string containing a qualified number.

    Parse a string containing a number followed by an optional multiplier character
    (%, B, K, M, G, T, P) in upper or lower case. Exit script with an error on a
    parse failure of any kind. The value may not be less than min_bytes if the
    multiplier character is not '%'. The value returned is normalized by indexing
    a multiplier table whose multiplier values are based on mfactor.

    Arguments:
        size      -- the quasi-numeric string to be parsed (e.g., '500G')
        min_bytes -- the minimum allowable size in bytes
        mfactor   -- the base multiplier to use in the mtable (1000 or 1024)

    Return an integer value between 1 and 100 if the size represents a percentage,
    or an integer value of 1 Mxbyte (10xx ** 2) or greater if the value represents
    a size in bytes (the API will not allow us to format the drive to less than
    1 MxByte anyway).
    """
    mtable = {'%': 1, 'B': mfactor ** 0, 'K': mfactor ** 1, 'M': mfactor ** 2, 'G': mfactor ** 3, 'T': mfactor ** 4, 'P': mfactor ** 5}
    if not size[-1] in string.digits:
        multiplier = size[-1].upper()
        sz = int(size[:-1])
    else:
        multiplier = 'B'
        sz = int(size)
    if not multiplier in mtable:
        print "Error parsing size '%s': unrecognized multiplier character '%s'." % (size, multiplier)
        exit(1)
    sz = sz * mtable[multiplier]
    if multiplier != '%' and sz < min_bytes:
        print "Error: Bad size value '%s'. Size values must be at least %d bytes." % (size, min_bytes)
        exit(1)
    elif multiplier == '%' and (size <= 0 or sz > 100):
        print "Error: Bad percent value '%s'. Percent values must be between 1 and 100." % size
        exit(1)
    return sz

def format(conn, iomp, blksz, mode, size):
    """Format the specified device to the specified size and block size.

    Format the specified device to either the default size and block size or
    to specified size and/or block size. If blksz is None and either size is
    None or 100% then call 'Format', otherwise call 'FormatSize' with specified
    values.

    Since 'FormatSize' formats to percentages of the MaxPhysicalCapacity rather
    then the MaxDefaultCapacity, this unfortunately means that the use of blksz
    changes the meaning of 100%. Thus, you may maximally overformat your drive
    by specifying any blksz value and a device-size of 100%.

    Arguments:
        conn  -- the connection on which to make the request
        iomp  -- the FIO_IoMemoryPort instance of the device to be formatted
        blksz -- the block size to use in bytes
        mode  -- 0 means size is percent, 1 means size is megabytes
        size  -- size as a percentage or in bytes, based on mode

    Returns (boolean, {}) - the value returned by conn.InvokeMethod()
    """
    if size == None or mode == mode_percent and size == 100:
        return conn.InvokeMethod('Format', iomp.path)
    else:
        if blksz == None:
            blksz = default_block_size
        if mode == mode_mbytes:
            size = size / one_mbyte
        return conn.InvokeMethod('FormatSize', iomp.path,
                BlockSize = pywbem.cim_types.Uint32(blksz),
                Mode = pywbem.cim_types.Uint16(mode),
                Size = pywbem.cim_types.Uint64(size))

def iomp_state(conn, iname):
    """Return the state of the last requested operation."""
    instance = conn.GetInstance(iname)
    return instance.properties['State'].value

# ---------------------------------------------------------------------------
#  START OF MAIN

# Setup argparse data structures and parse command-line arguments

description = '''Fusion-io %(prog)s utility (3.1.4.111 tahoe@a2a30d5308e5M).
Copyright 2006-2011 Fusion-io, Inc. All rights reserved.'''

epilog = '''B, K, M, G, T, P, and % are optional <u>nit indicators (meaning Bytes, KBytes,
MBytes, GBytes, TBytes, PBytes and percent) that may be used with all options that
accept a size except --block-size, which only accepts B or K. Units are assumed
to be Bytes if not specified. Qualifiers are not case sensitive.'''

parser = argparse.ArgumentParser(prog='fio-format', description=description, formatter_class=argparse.RawDescriptionHelpFormatter, epilog=epilog)
parser.add_argument('device', metavar='<device>', nargs=1, help='a pathname to the device (/dev/fctX)');
parser.add_argument('-b', '--block-size', metavar='<size>[B,K]', help='set the block size in Bytes or KiBytes (base 2)')
parser.add_argument('-s', '--device-size', metavar='<size>[<u>]', help='size to format device; max size is default capacity')

#Note: overformat not currently implemented in CIM interface
#parser.add_argument('-o', '--overformat', metavar='<size>[<u>]', help='overformat size; max size is max physical capacity')

parser.add_argument('-q', '--quiet', action='store_true', dest='quiet_flag', help='don\'t display progress')
parser.add_argument('-y', '--yes', action='store_true', dest='yes_flag', help='auto answer \'yes\' to all confirmation requests')
parser.add_argument('-v', '--version', action='version', version='%(prog)s version 3.1.4.111 tahoe@a2a30d5308e5M', help="print version and exit")

wbemgroup = parser.add_argument_group('remote options')
wbemgroup.add_argument('--timeout', metavar='<seconds>', default='60', dest='timeout', type=int, help='seconds to wait for request to complete (default:60; a timeout value of 0 means don\'t wait)')
wbemgroup.add_argument('--server', metavar='<address>', dest='server', required=True, help='remote host address/dns name')
wbemgroup.add_argument('--namespace', metavar='<namespace>', dest='namespace', default='fio/cimv2', help='CIM namespace (default:fio/cimv2)')
wbemgroup.add_argument('--username', metavar='<username>', dest='username', default='anonymous', help='remote user (default:anonymous)')
wbemgroup.add_argument('--password', metavar='<password>', dest='password', default='', help="remote user's password (default:none)")
wbemgroup.add_argument('--no-ssl', default=True, action='store_false', dest='secure_flag', help='disable secure http on connection');

args = parser.parse_args()

# parse path name (/dev/fctx); extract fctx

short_device = string.split(args.device[0], '/')[-1]

# build a URL and credentials tuple from command line arguments

url = make_wbem_url(args)
creds = (args.username, args.password) if args.secure_flag else None

# establish connection from the URL

conn = pywbem.WBEMConnection(url, creds, default_namespace=args.namespace);

# check if remote host is an ESX system, we only use fio/cimv2 namespace on ESX

ESX = (args.namespace == 'fio/cimv2')

# FIO_DeviceSAPImplementation maps FIO_IoMemoryPort (/dev/fctX) to FIO_ProtocolEndpoint
# (/dev/fioY). Enumerate all FIO_DeviceSAPImplementation relations. Locate the
# relation that maps the specified hardware device to the desired FIO_LogicalSSD.
# Request Associated instances of that FIO_ProtocolEndpoint and search the returned
# list for the related FIO_LogicalSSD.
# HACK: When multiple devices are detached, the 'Name' field of their LogicalSSD instances
# are the same, so the CIM cannot distinguish between them.  Take the first one we find,
# and let the provider internally handle any errors that might arise if there are differences
# in min/max/default block size

lssd = iomp = None
dsi_inames = conn.EnumerateInstanceNames('FIO_DeviceSAPImplementation')
for iname in dsi_inames:
    if iname['Antecedent'].keybindings['DeviceID'] == short_device:
        iomp = conn.GetInstance(iname['Antecedent'])
        assocs = conn.Associators(iname['Dependent'])
        for assoc in assocs:
            if assoc.classname == 'FIO_LogicalSSD':
                lssd = assoc
            if lssd != None and iomp != None:
                break
        break

if lssd == None or iomp == None:
    print >> sys.stderr, "Error: Unable to locate device '%s'." % args.device[0]
    exit(1)

# Access desired block-size properties (use global defaults if values are None)

logical_device_name = lssd.properties['Name'].value
min_block_size = lssd.properties['MinBlockSize'].value if lssd.properties['MinBlockSize'].value != None else MIN_BLOCK_SIZE
max_block_size = lssd.properties['MaxBlockSize'].value if lssd.properties['MaxBlockSize'].value != None else MAX_BLOCK_SIZE
default_block_size = lssd.properties['DefaultBlockSize'].value if (lssd.properties['DefaultBlockSize'].value != None and not ESX) else DEFAULT_BLOCK_SIZE
p2bs_required = lssd.properties['PowerOfTwoBlockSize'].value if lssd.properties['PowerOfTwoBlockSize'].value != None else False
blksz_multiplier = lssd.properties['BlockSizeMultiplier'].value if lssd.properties['BlockSizeMultiplier'].value != None else 1

# Parse values in unitized fields (block_size, overformat, device_size)

block_size = parse_size(args.block_size, min_block_size, 1024) if args.block_size != None else None
#overformat = parse_size(args.overformat, one_mibyte, 1000) if args.overformat != None else None
device_size = parse_size(args.device_size, one_mibyte, 1000) if args.device_size != None else None
mode = mode_percent if device_size <= 100 else mode_mbytes

# Verify device is not attached before beginning

if iomp_state(conn, iomp.path) == st_Attached:
    print "Device %s is attached.  You must detach the device before formatting" % short_device
    exit(1)

# Verify specified block size

if block_size != None and block_size != default_block_size:
    if ESX:
        print "Error: ESX only supports 512 byte block size"
        exit(1)
    if p2bs_required == True and block_size & 1 == 1:
        print "Error: Bad block size '%d'. Block size must be a power of 2 on device '%s'." \
                % (block_size, args.device[0])
        exit(1)
    elif blksz_multiplier != None and block_size % blksz_multiplier != 0:
        print "Error: Bad block size '%d'. Block size must be a multiple of %d on device '%s'." \
                % (block_size, blksz_multiplier, args.device[0])
        exit(1)
    elif block_size < min_block_size or block_size > max_block_size:
        print "Error: Bad block size '%d'. Block size must be within range [%d .. %d] bytes on device '%s'." \
                % (block_size, min_block_size, max_block_size, args.device[0])
        exit(1)

# Indicate specified operation if quiet_flag is false

if not args.quiet_flag:
    print "Formatting '%s' to" % args.device[0],
    if device_size == None:
        print "default size.",
    elif mode == mode_percent:
        print "%d%% capacity." % device_size,
    else:
        print "%.2f GBytes (%.2f GiBytes)." % (float(device_size) / one_gbyte,
                float(device_size) / one_gibyte),
    print "Using a %sblock (sector) size of %s bytes." % \
            ("default " if block_size == None or block_size == default_block_size else "",
                    block_size if block_size else default_block_size)

# Check for overformat request here...

# Verify specified operation if yes_flag is false

if not args.yes_flag:
    print "WARNING: Formatting will destroy any existing data on the device!"
    answer = raw_input("Do you wish to continue? [y/N] ")
    if answer[0] != 'y' and answer[0] != 'Y':
        print "Aborted."
        exit(1)

# Houston, we have a green light

rv = format(conn, iomp, block_size, mode, device_size)

if not rv[0]:
    print >> sys.stderr, "Format request failed,", rv
    exit(1)

# Wait for operation to start, then wait the desired number of seconds for it to complete

state = iomp_state(conn, iomp.path)
while state != st_Busy and state != st_Formatting:
    time.sleep(1)
    state = iomp_state(conn, iomp.path)

timeout = args.timeout
if not args.quiet_flag and args.timeout != 0: 
    print "Formatting",
    sys.stdout.flush()
while timeout > 0 and (state == st_Busy or state == st_Formatting):
    time.sleep(1)
    if not args.quiet_flag: 
        print ".",
        sys.stdout.flush()
    state = iomp_state(conn, iomp.path)
    timeout = timeout - 1

# check for success, failure, or timeout

if not args.quiet_flag and args.timeout != 0:
    if state == st_Failed:
        print "failed."
        exit(1)
    elif state != st_Busy and state != st_Formatting:
        print "complete."
        exit(0)
    else:
        print "request timed out."
        exit(-1)

# END OF MAIN
# ---------------------------------------------------------------------------
