#!/usr/bin/env python
#
# fio-status.py     Emulate Fusion-io fio-status command-line utility
#

import sys
if sys.hexversion < 0x02060500:
    print >> sys.stderr, "This script requires python 2.6.5 or higher."
    exit(1)

try:
    import argparse
except ImportError:
    print >> sys.stderr, "Please install argparse from http://code.google.com/p/argparse."
    exit(1)

try:
    import pywbem
except ImportError:
    print >> sys.stderr, "Please install the pywbem module from http://sourceforge.net/projects/pywbem."
    exit(1)

import string

__author__ = "Fusion-io management software team"
__date__   = "$Feb 14, 2011 10:20:55 AM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

same_line_sep = ", "

def make_wbem_url(args):
    """Build and return an http or https url based on args."""
    return "http%s://%s" % ('s' if args.secure_flag else '', args.server)

def get_system_name(conn):
    """Return the system name for the specified connection."""
    computer_system = conn.EnumerateInstanceNames("FIO_ComputerSystem")
    return computer_system[0].keybindings["Name"]

def get_driver_version_info(conn):
    """Return the driver FIO_SoftwareIdentity instance driver for conn."""
    system_name = get_system_name(conn)
    versions = conn.EnumerateInstanceNames("FIO_SoftwareIdentity")
    for version in versions:
        if version.keybindings['InstanceID'] == 'Fusion-io:{0}:driver'.format(system_name):
            return conn.GetInstance(version)

def get_iodimm_info(conn, device):
    """Gather and return information about a specific iodimm or all of them.

    Gather and combine information for either a specific iodimm or all iodimms
    in the system by querying for one or all FIO_IoMemoryPort instances and
    zipping them up with associated FIO_LogicalSSD, FIO_StorageExtent,
    FIO_PCIDevice, FIO_PhysicalPackage, FIO_SoftwareIdentity,
    FIO_TemperatureSensor, and FIO_SSDStatistics instances.

    Arguments:
      conn -- the connection to query
      device -- the device to query for (or None)

    Return a zipped list of tuples --
    [
        (
            FIO_IoMemoryPort,
            FIO_LogicalSSD,
            FIO_StorageExtent,
            FIO_PCIDevice,
            FIO_PhysicalPackage,
            FIO_SoftwareIdentity,
            FIO_TemperatureSensor,
            FIO_PowerSensor(voltage)
            FIO_PowerSensor(current)
            FIO_PowerSensor(power)
            FIO_SSDStatistics
        ),
        ...
    ]
    """
    # enumerate or get specific FIO_IoMemoryPort instance
    if device == None:
        iomps = conn.EnumerateInstances("FIO_IoMemoryPort")
    else:
        short_device = string.split(device, '/')[-1]
        inames = conn.EnumerateInstanceNames("FIO_IoMemoryPort")
        iomps=[]
        for iname in inames:
            if iname.keybindings[u'DeviceID'] == short_device:
                iomps = [conn.GetInstance(iname)]
                break

    # for each iomp found, get associated FIO_PCIDevice,
    # FIO_ProtocolEndpoint, and FIO_SSDStatistics instances
    pcids = []
    pepts = []
    stats = []
    for iomp in iomps:
        assocs = conn.Associators(iomp.path)
        found_pcid = found_pept = found_stat = False;
        for assoc in assocs:
            if assoc.classname == 'FIO_PCIDevice':
                pcids.append(assoc)
                found_pcid = True
            elif assoc.classname == 'FIO_ProtocolEndpoint':
                pepts.append(assoc)
                found_pept = True
            elif assoc.classname == 'FIO_SSDStatistics':
                stats.append(assoc)
                found_stat = True
        if not found_pcid:
            pcids.append(None)
        if not found_pept:
            pepts.append(None)
        if not found_stat:
            stats.append(None)

    # for each pcid, get the associated FIO_PhysicalPackage and
    # FIO_SoftwareIdentity instances
    ppkgs = []
    swids = []
    for pcid in pcids:
        assocs = conn.Associators(pcid.path)
        found_ppkg = found_swid = False;
        for assoc in assocs:
            if assoc.classname == 'FIO_PhysicalPackage':
                ppkgs.append(assoc)
                found_ppkg = True;
            elif assoc.classname == 'FIO_SoftwareIdentity':
                swids.append(assoc)
                found_swid = True;
        if not found_ppkg:
            ppkgs.append(None)
        if not found_swid:
            swids.append(None)

    # for each pept, get the associated FIO_LogicalSSD instance
    lssds = []
    for pept in pepts:
        assocs = conn.Associators(pept.path)
        for assoc in assocs:
            if assoc.classname == 'FIO_LogicalSSD':
                lssds.append(assoc)
                break;
        else:
            lssds.append(None)

    # for each lssd, get the associated FIO_StorageExtent instance
    sexts = []
    for lssd in lssds:
        assocs = conn.Associators(lssd.path)
        for assoc in assocs:
            if assoc.classname == 'FIO_StorageExtent':
                sexts.append(assoc)
                break;
        else:
            sexts.append(None)

    # for each ppkg, get the associated FIO_TemperatureSensor and FIO_PowerSensor instances
    # power sensors are separated by type
    tsens = []
    psensV = []
    psensA = []
    psensW = []
    for ppkg in ppkgs:
        assocs = conn.Associators(ppkg.path)
        found_tsen = found_psenV = found_psenA = found_psenW = False;
        for assoc in assocs:
            if assoc.classname == 'FIO_TemperatureSensor':
                tsens.append(assoc)
                found_tsen = True;
            elif assoc.classname == 'FIO_PowerSensor':
                sensor_type = string.split(assoc['DeviceID'], ':')[-1]
                if sensor_type == 'voltage':
                    psensV.append(assoc)
                    found_psenV = True;
                elif sensor_type == 'current':
                    psensA.append(assoc)
                    found_psenA = True;
                else:
                    psensW.append(assoc)
                    found_psenW = True;
        if not found_tsen:
            tsens.append(None)
        if not found_psenV:
            psensV.append(None)
        if not found_psenA:
            psensA.append(None)
        if not found_psenW:
            psensW.append(None)

    # zip'em all up and return one list of 5-tuples
    return zip(iomps, lssds, sexts, pcids, ppkgs, swids, tsens, psensV, psensA, psensW, stats)

def display_summary_info(iodimm_info, driver_version, device):
    """Display summary header information about specific or all iodimms/drivers

    Display summary information about the specified iodimm, or about all iodimms
    in the system if none was specified. The iodimm_info argument will contain
    just the iodimm specified or all if none was specified. The driver_version
    argument contains information about the version of the driver on this system.
    The device argument is None if none was specified.

    Arguments:
      iodimm_info -- a list of one or more iodimms installed or specified
      driver_version -- information about the installed driver
      device -- the device specified, or None of none was specified
    """
    # adapter types and tallies (AdapterType is array index)
    tallies = [["Single Adapter", 0], ["Dual Adapter", 0], ["ioSan", 0], ["ioOctal Base", 0],
               ["ioOctal Mezz", 0], ["ioDimm", 0], ["ioMono", 0], ["ioXtreme", 0], ["HP Mezz", 0],
               ["ioDrive Controller", 0], ["HP Adrenaline 3", 0], ["2.5 inch Fused", 0], ["NAND Module", 0],
               ["Dual Controller Adapter", 0], ["Single Controller Adapter", 0]]

    # sum up special drive types (AdapterType value is array index)
    total = 0
    for iodimm in iodimm_info:
        iodimm_pci_dict = dict(iodimm[3])
        atype = iodimm_pci_dict['AdapterType']
        if atype >= 0 and atype < len(tallies):
            tallies[atype][1] += 1
            total += 1

    # first header line
    print "Found %s ioMemory Device%s in this system" % (len(iodimm_info), 's' if len(iodimm_info) != 1 else ''),
    if total:
        print "with",
        for tally in tallies:
            if tally[1]:
                print "%d %s%s" % (tally[1], tally[0], 's' if tally[1] != 1 else ''),

    if args.device != None:
        print "as device '%s'" % args.device,

    print ""    # terminate the line started above

    # second header line
    if driver_version != None:
        dvitems = dict(driver_version.items())
        print 'Fusion-io driver version: %s build %s' % (dvitems[u'VersionString'],
                dvitems[u'BuildNumber'] if dvitems[u'BuildNumber'] != None else '<unknown>')

def safe_convert_to_int(x):
    """Convert string to int; convert non-integer strings to -1"""
    try: return int(x)
    except ValueError: return -1

def get_property(pclass, properties, propname):
    """Convert a property value to a string via the class value map.

    Convert a property value to its corresponding string value as specified by
    the class value map from which the propery was taken. If the class doesn't
    have a value map for the target property, just use the value directly. If
    the value is a list, convert the list to a reasonable string. If there are
    units involved, append the unit specifier to the value string.

    Arguments:
      pclass -- the CIMClass of the target property
      properties -- the property dictionary
      propname -- the name of the target property
    """
    if 'ValueMap' in pclass.properties[propname].qualifiers:
        # create mapping from value to string
        value_map = dict(zip(map(safe_convert_to_int,
                pclass.properties[propname].qualifiers[u'ValueMap'].value),
                pclass.properties[propname].qualifiers[u'Values'].value))

        # get the property value - it may be a list - if not, convert to list
        prop_value = properties[propname]
        if type(prop_value) != type(list()):
            prop_value = [prop_value]

        # iterate though each item in the list, building a string of values
        value = ""
        for pval in prop_value:
            value += value_map[pval] + ", "
        if len(value):
            value = value[:-2]

    else:
        value = properties[propname]

    if 'Units' in pclass.properties[propname].qualifiers:
        value = str(value) + " " + str(pclass.properties[propname].qualifiers['Units'].value);

    return value;

def print_all_displayable_properties(pclass, props, filter, deprec = []):
    """Print all displayable desired properties in the given property map.

    Print all properties in the specified property map that are not deprecated
    or filtered out. Print the property name left-justified in a 30 column field
    followed by a colon and the property value. Filtering is done at three levels:
    The first level is whether the property has a value. Valueless properties are
    never printed. The second level is always filtered and consists of names that
    are part of the CIM base schema. The third level is done if the --all flag
    is not specified on the command line and consists of more esoteric
    information about the device.

    Arguments:
      pclass -- the class containing meta information about the property value
      props -- the property dictionary to be displayed
      filter -- a list of property names to be suppressed
      deprec -- a list of deprecated properties that should be displayed
    """
    always_filter = ['CreationClassName', 'SystemName', 'Name', 'ElementName',
            'Description', 'InstanceID', 'Caption', 'DeviceID', 'UUID',
            'SystemCreationClassName']
    name = ": " + props['Name'] if 'Name' in props else ''
    print "\n  %s%s" % (pclass.classname[4:], name)
    for key in props.keys():
        deprecated = 'Deprecated' in pclass.properties[key].qualifiers and not key in deprec
        filtered = key in always_filter or key in filter
        if props[key] != None and not deprecated and not filtered:
            print "\t%-30s: %s" % (key, get_property(pclass, props, key))

def display_iodimm_info(conn, iodimm_info, display_list, more):
    """Display adapter and iodimm info for a specific iodimm or all iodimms.

    Our CIM model doesn't break out adapters from iodimms. A FIO_IoMemoryPort
    instance contains all iodimm and adapter information, so if an actual
    physical adapter contains two iodimms then both FIO_IoMemoryPort instances
    contain the same adapter information. While we could figure out physical
    adapters by comparing field information, though redundant, it's been decided
    to simply display all information (adaptor and iodimm) for each iodimm in
    the system.

    Arguments:
      conn -- the connection from which to get class information
      iodimm_info -- the info for all iodimms or a specific iodimm if device was specified
      display_list -- whether to display the info as a list based on CIM hierarchy
      more -- display more info if true
    """

    # get classes for property qualifiers
    iomp_class = conn.GetClass('FIO_IoMemoryPort')
    lssd_class = conn.GetClass('FIO_LogicalSSD')
    sext_class = conn.GetClass('FIO_StorageExtent')
    pcid_class = conn.GetClass('FIO_PCIDevice')
    ppkg_class = conn.GetClass('FIO_PhysicalPackage')
    swid_class = conn.GetClass('FIO_SoftwareIdentity')
    tsen_class = conn.GetClass('FIO_TemperatureSensor')
    psen_class = conn.GetClass('FIO_PowerSensor')
    stat_class = conn.GetClass('FIO_SSDStatistics')

    # filter these out unless 'more' flag is specified
    iomp_filter = [
            'RequestedState', 'FlashbackAvailability', 'WriteRegulationLevelLifespan',
            'TransitioningToState', 'EnabledDefault', 'WriteRegulationLevelThermal',
            'LifespanExtenderState', 'WriteRegulationLevelPower', 'PortType',
            'WriteRegulationLevelActual', 'OtherPortType', 'PowerlossProtectDisabled']
    lssd_filter = [
            'RequestedState', 'Capabilities', 'TransitioningToState',
            'EnabledDefault', 'DefaultBlockSize', 'MinBlockSize', 'PowerOfTwoBlockSize',
            'BlockSizeMultiplier', 'MaxBlockSize']
    sext_filter = [
            'RequestedState', 'IsBasedOnUnderlyingRedundancy', 'Primordial',
            'TransitioningToState', 'EnabledDefault', 'NameFormat', 'IsComposite',
            'ExtentStatus'
            ]
    pcid_filter = [
            'Slot', 'RequestedState', 'DomainNumber', 'TransitioningToState',
            'PowerlossProtectionEnabled', 'PowerlossProtectionAvailable',
            'PowerMonitorEnabled', 'DeviceNumber', 'FunctionNumber',
            'EnabledDefault', 'ProtocolSupported', 'BusNumber', 'ClassCode',
            'PowerCompatibility']
    ppkg_filter = [
            'SKU', 'PartNumber', 'CanBeFRUed', 'SparePartNumber',
            'AssemblyNumber', 'Manufacturer', 'Tag']
    swid_filter = [
            'isEntity', 'IsLargeBuildNumber', 'Manufacturer', 'Classifications']
    tsen_filter = [
            'TransitioningToState', 'EnabledDefault', 'RequestedState',
            'PossibleStates', 'SensorContext', 'SupportedThresholds',
            'EnabledThresholds', 'Hysteresis']
    tsens_deprecated = ['CurrentReading']   # display these fields anyway
    psen_filter = [
            'TransitioningToState', 'EnabledDefault', 'RequestedState',
            'PossibleStates', 'Supported Thresholds', 'EnabledThresholds',
            'Hysteresis', 'CommunicationStatus', 'NormalMax', 'NormalMin']
    stat_filter = [
            'TotalDataMByteCapacity', 'UsableDataMByteCapacity', 'SampleInterval']

    for iodimm in iodimm_info:

        # create property field dictionaries for this iodimm
        iomp_props = dict(iodimm[0])
        lssd_props = dict(iodimm[1])
        sext_props = dict(iodimm[2])
        pcid_props = dict(iodimm[3])
        ppkg_props = dict(iodimm[4])
        swid_props = dict(iodimm[5])
        tsen_props = dict(iodimm[6])
        psenV_props = dict(iodimm[7])
        psenA_props = dict(iodimm[8])
        psenW_props = dict(iodimm[9])
        stat_props = dict(iodimm[10])

        if display_list:
            # print all displayable properties for all class instances we have
            print "\n/dev/%s:" % iomp_props['DeviceID']
            print_all_displayable_properties(iomp_class, iomp_props, [] if more else iomp_filter)
            print_all_displayable_properties(lssd_class, lssd_props, [] if more else lssd_filter)
            print_all_displayable_properties(sext_class, sext_props, [] if more else sext_filter)
            print_all_displayable_properties(pcid_class, pcid_props, [] if more else pcid_filter)
            print_all_displayable_properties(ppkg_class, ppkg_props, [] if more else ppkg_filter)
            print_all_displayable_properties(swid_class, swid_props, [] if more else swid_filter)
            print_all_displayable_properties(tsen_class, tsen_props, [] if more else tsen_filter, tsens_deprecated)
            print_all_displayable_properties(psen_class, psenV_props, [] if more else psen_filter)
            print_all_displayable_properties(psen_class, psenA_props, [] if more else psen_filter)
            print_all_displayable_properties(psen_class, psenW_props, [] if more else psen_filter)
            print_all_displayable_properties(stat_class, stat_props, [] if more else stat_filter)

        else:
            # print all information in user-readable format akin to fio-status utility
            print ""
            print_adapter_header(pcid_class, pcid_props)
            print_adapter_base_info(pcid_class, pcid_props, ppkg_class, ppkg_props)
            if more:
                print_adapter_extra_info(ppkg_class, ppkg_props)
                print_pci_power_info(psen_class, psenV_props, psenA_props, psenW_props)
                print_pci_bus_info(pcid_class, pcid_props)
            print ""
            print_iodimm_header(iomp_class, iomp_props, sext_class, sext_props)
            print_iodimm_base_info(ppkg_class, ppkg_props, pcid_class, pcid_props)
            if more:
                print_iodimm_extra_info(pcid_class, pcid_props)
            print_firmware_info(swid_class, swid_props)
            print_format_base_info(stat_class, stat_props)
            if more:
                print_format_extra_info(sext_class, sext_props)
            print_temperature_info(tsen_class, tsen_props)
            if more:
                print_lifetime_data_info(stat_class, stat_props)
                print_ram_usage_info(stat_class, stat_props)

def print_adapter_header(pcid_class, pcid_props):
    if pcid_props['AdapterType'] != None:
        print "Adapter: %s" % get_property(pcid_class, pcid_props, 'AdapterType')
    else:
        print "Unknown Adapter:"

def print_adapter_base_info(pcid_class, pcid_props, ppkg_class, ppkg_props):
    pre = "\t"
    if print_if_exists(ppkg_class, ppkg_props, 'Model', pre):
        pre = same_line_sep
    if print_if_exists(ppkg_class, ppkg_props, 'SKU', pre + "Product Number: "):
        pre = same_line_sep
    print_if_exists(pcid_class, pcid_props, 'AdapterUuid', pre + "SN: ")
    print ""

def print_adapter_extra_info(ppkg_class, ppkg_props):
    pre = "\t"
    if print_if_exists(ppkg_class, ppkg_props, 'AssemblyNumber', pre):
        pre = same_line_sep
    print_if_exists(ppkg_class, ppkg_props, 'SKU', pre + "PN: ")
    print ""

def print_pci_power_info(psen_class, psenV_props, psenA_props, psenW_props):
    if psenV_props['CurrentReading'] != None and psenV_props['MinimumReading'] != None and psenV_props['PeakReading'] != None:
        print "\tPCIe Bus Voltage(mV): current: %s, min: %s, max: %s" % (get_property(psen_class, psenV_props, 'CurrentReading'),
            get_property(psen_class, psenV_props, 'MinimumReading'), get_property(psen_class, psenA_props, 'PeakReading'))
    if psenA_props['CurrentReading'] != None and psenA_props['PeakReading'] != None:
        print "\tPCIe Bus Current(mA): current: %s, max: %s" % (get_property(psen_class, psenA_props, 'CurrentReading'),
            get_property(psen_class, psenA_props, 'PeakReading'))
    if psenW_props['CurrentReading'] != None and psenW_props['PeakReading'] != None:
        print "\tPCIe Bus Power(mW): current: %s, max: %s" % (get_property(psen_class, psenW_props, 'CurrentReading'),
            get_property(psen_class, psenW_props, 'PeakReading'))

def print_pci_bus_info(pcid_class, pcid_props):
    if pcid_props['PowerLimitWatts'] != None:
        print "\tPCIe slot available power(W): %s" % get_property(pcid_class, pcid_props, 'PowerLimitWatts')
    pre = "\tPCIe Negotiated Link: "
    if print_if_exists(pcid_class, pcid_props, 'LinkWidth', pre, " lanes"):
        pre = same_line_sep
    if print_if_exists(pcid_class, pcid_props, 'LinkSpeed', pre, " GT/sec"):
        pre = same_line_sep
    print_if_exists(pcid_class, pcid_props, 'BandwidthMBps', pre, " MBytes total link speed")
    print ""

def print_iodimm_header(iomp_class, iomp_props, sext_class, sext_props):
    if iomp_props['DeviceID'] != None:
        print get_property(iomp_class, iomp_props, 'DeviceID') + ":",
    if iomp_props['State'] != None:
        state = get_property(iomp_class, iomp_props, 'State')
        if state == "Attached" and sext_props['Name'] != None:
            print " Attached as %s" % get_property(sext_class, sext_props, 'Name'),
        elif state == "Minimal" and iomp_props['MinimalModeReason'] != None:
            print " Minimal Mode: %s" % get_property(iomp_class, iomp_props, 'MinimalModeReason'),
        else:
            print state,
    print ""

def print_iodimm_base_info(ppkg_class, ppkg_props, pcid_class, pcid_props):
    if ppkg_props['SerialNumber'] != None:
        print "\tSN: %s" % (get_property(ppkg_class, ppkg_props, 'SerialNumber'))
    pre = "\tPCI "
    if print_if_exists(pcid_class, pcid_props, 'Name', pre + "Addr: "):
        pre = same_line_sep
    print_if_exists(pcid_class, pcid_props, 'Slot', pre + "Slot: ")
    print ""


def print_iodimm_extra_info(pcid_class, pcid_props):
    pre = "\tPowerloss Protection "
    if print_if_exists(pcid_class, pcid_props, 'PowerlossProtectionAvailable', pre + "Available: "):
        pre = same_line_sep
    print_if_exists(pcid_class, pcid_props, 'PowerlossProtectionEnabled', pre + "Enabled: ")
    print ""
    pre = "\tPCI "
    if print_if_exists(pcid_class, pcid_props, 'VendorID', pre + "Vendor: "):
        pre = same_line_sep
    if print_if_exists(pcid_class, pcid_props, 'PCIDeviceID', pre + "Device: "):
        pre = same_line_sep
    if print_if_exists(pcid_class, pcid_props, 'SubsystemVendorID', pre + "Sub-vendor: "):
        pre = same_line_sep
    print_if_exists(pcid_class, pcid_props, 'SubsystemID', pre + "Sub-device: ")
    print ""

def print_firmware_info(swid_class, swid_props):
    pre = "\tFirmware "
    if print_if_exists(swid_class, swid_props, 'VersionString', pre + "Version: "):
        pre = same_line_sep
    print_if_exists(swid_class, swid_props, 'LargeBuildNumber', pre + "Revision: ")
    print ""

def print_format_base_info(stat_class, stat_props):
    if stat_props['TotalLogicalMByteCapacity'] != None:
        print "\t%s MBytes device size" % get_property(stat_class, stat_props, 'TotalLogicalMByteCapacity')

def print_format_extra_info(sext_class, sext_props):
    if sext_props['NumberOfBlocks'] != None and sext_props['BlockSize'] != None:
        print "\tFormat: %s sectors of %s" % (get_property(sext_class, sext_props, 'NumberOfBlocks'), get_property(sext_class, sext_props, 'BlockSize'))

def print_temperature_info(tsen_class, tsen_props):
    if tsen_props['CurrentReading'] != None:
        print "\tInternal temperature: %s degC" % get_property(tsen_class, tsen_props, 'CurrentReading')

def print_lifetime_data_info(stat_class, stat_props):
    if stat_props['PhysicalMBytesWritten'] != None and stat_props['PhysicalMBytesRead'] != None:
        print "\tLifetime Data Volumes:"
        print "\t\tPhysical MBytes Written: %s" % get_property(stat_class, stat_props, 'PhysicalMBytesWritten')
        print "\t\tPhysical MBytes Read: %s" % get_property(stat_class, stat_props, 'PhysicalMBytesRead')

def print_ram_usage_info(stat_class, stat_props):
    if stat_props['CurrentMByteRAMUsage'] != None and stat_props['PeakMByteRAMUsage'] != None:
        print "\tRAM usage:"
        print "\t\tCurrent: %s MBytes" % get_property(stat_class, stat_props, 'CurrentMByteRAMUsage')
        print "\t\tPeak: %s MBytes" % get_property(stat_class, stat_props, 'PeakMByteRAMUsage')

def print_if_exists(pclass, props, key, pre, post = ''):
    if props[key] != None:
        print "%s%s%s" % (pre, get_property(pclass, props, key), post),
        return True
    return False

# ---------------------------------------------------------------------------
# START OF MAIN

# Setup argparse data structures and parse command-line arguments

description = '''Fusion-io %(prog)s utility version 3.1.4.111 tahoe@a2a30d5308e5M.
  General information utility for Fusion-io devices.
  Copyright 2006-2011 Fusion-io, Inc. All rights reserved.'''

parser = argparse.ArgumentParser(prog='fio-status', description=description, formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument('device', metavar='<device>', nargs='?', help='a pathname to the device (/dev/fctX)');
parser.add_argument('-v', '--version', action='version', version='%(prog)s version 3.1.4.111 tahoe@a2a30d5308e5M', help="print version and exit")

fiogroup = parser.add_argument_group('device options')
fiogroup.add_argument('-a', '--all', action='store_true', dest='all_flag', help='report all available information')
fiogroup.add_argument('-c', '--count', action='store_true', dest='count_flag', help='report the number of Fusion-io boards installed')
fiogroup.add_argument('-l', '--list', action='store_true', dest = 'list_flag', help='list all data according to CIM hierarchy format')
#fiogroup.add_argument('-d', '--data-volume', action='store_true', dest='data_volume_flag', help='report the volume of data read and written')
#fiogroup.add_argument('-f', '--format-key', action='store_true', dest='format_key_flag', help="force alternate 'key=value' format output")

wbemgroup = parser.add_argument_group('remote options')
wbemgroup.add_argument('--server', metavar='<address>', dest='server', required=True, help='remote host address/dns name')
wbemgroup.add_argument('--namespace', metavar='<namespace>', dest='namespace', default='fio/cimv2', help='CIM namespace (default:fio/cimv2)')
wbemgroup.add_argument('--username', metavar='<username>', dest='username', default='anonymous', help='remote user (default:anonymous)')
wbemgroup.add_argument('--password', metavar='<password>', dest='password', default='', help="remote user's password (default:none)")
wbemgroup.add_argument('--no-ssl', default=True, action='store_false', dest='secure_flag', help='disable secure http on connection');

args = parser.parse_args()

# build a URL and credentials tuple from command line arguments

url = make_wbem_url(args)
creds = (args.username, args.password) if args.secure_flag else None

# establish connection from the URL

conn = pywbem.WBEMConnection(url, creds, default_namespace=args.namespace);

# gather required data

iodimm_info = get_iodimm_info(conn, args.device)
driver_version = get_driver_version_info(conn)

# first check for --count flag - if given, just print count and exit

if args.count_flag:
    print len(iodimm_info)
    exit(0)

# display summary header information

display_summary_info(iodimm_info, driver_version, args.device)

# display information about each adapter and drive (some or all)

display_iodimm_info(conn, iodimm_info, args.list_flag, args.all_flag)

# END OF MAIN
# ---------------------------------------------------------------------------

