#!/usr/bin/env python
#
# fio-update-iodrive.py	Emulate Fusion-io fio-update-iodrive command-line utility
#

import sys
if sys.hexversion < 0x02060500:
    print >> sys.stderr, "This script requires python 2.6.5 or higher."
    exit(1)

try:
    import argparse
except ImportError:
    print >> sys.stderr, "Please install argparse from http://code.google.com/p/argparse."
    exit(1)

try:
    import pywbem
except ImportError:
    print >> sys.stderr, "Please install the pywbem module from http://sourceforge.net/projects/pywbem." 
    exit(1)

import string
import time

__author__ = "Fusion-io management software team"
__date__ ="$Feb 15, 2011 1:42:17 PM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

# SoftwareInstallationService states

st_Complete = 0
st_Unsupported = 1
st_Error = 2
st_Timeout = 3
st_Failed = 4
st_Inputerror = 5
st_Busy = 6
st_Started = 4096

# ConcreteJob states

job_New = 2
job_Starting = 3
job_Running = 4
job_Suspended = 5
job_Ending = 6
job_Complete = 7
job_Terminated = 8
job_Killed = 9
job_Error = 10

# IoMemoryPort states

iomp_Unknown = 0
iomp_Detached = 1
iomp_Attaching = 2
iomp_Scanning = 3
iomp_Attached = 4
iomp_Detaching = 5
iomp_Stopping = 6
iomp_Stopped = 7
iomp_Formatting = 8
iomp_Minimal = 9
iomp_Failed = 10
iomp_Updating = 11
iomp_Busy = 12

def make_wbem_url(args):
    return "{0}://{1}".format('https' if args.secure_flag else 'http', args.server)

def get_installation_service(conn, device):
    """Return the installation service for the specified device
    Each FIO_SoftwareInstallationService instance is mapped to a specific
    device and can only create an update operation for that device
    """
    services = conn.EnumerateInstanceNames('FIO_SoftwareInstallationService')
    for service in services:
        if service.keybindings['Name'] == device:
            return conn.GetInstance(service)
    return None

def get_version_list(iservice):
    """Return the installation service's version list as a list of 2-tuples.
    Each 2-tuple contains: (<version-string>, <fff file>).
    Return a list of version 2-tuples or an empty list.
    """
    vlist_str = iservice.properties['AvailableVersions'].value
    if vlist_str != None:
        version_list = vlist_str.split(',')
        return [tuple(version.split(':')) for version in version_list]
    return []

def get_directory(iservice):
    """Return the installation service's current configured firmware directory """
    return iservice.properties['FirmwareDirectory'].value

def get_specific_version(iservice, version):
    """Return a version 2-tuple specified by version string or by .fff file."""
    vlist = get_version_list(iservice)
    for v in vlist:
        if v[0] == version or v[1] == version:
            return v

def display_version_list(vlist):
    """Enumerate and display the specified list of versions.
    NOTE: vlist must contain at least one item.
    """
    print "\n",
    print "     Available software updates"
    print "     --------------------------"
    for version in enumerate(vlist):
        print "[%2d] %-12s (%s)" % (version[0], version[1][0], version[1][1])

def prompt_for_version(iservice):
    """Display a version list and prompt for a choice.
    Return the selected version tuple (version, fff-file)
    """
    vlist = get_version_list(iservice)
    display_version_list(vlist)
    while len(vlist):
        selection = raw_input("\nPlease select a version by number: ")
        try:
            iselection = int(selection)
            if iselection >= 0 and iselection < len(vlist):
                return vlist[iselection]
        except ValueError:
            pass
        print "Not a valid selection. Please try again."
        display_version_list(vlist)

def build_device_dictionary(conn, sdevice):
    """Return a dictionary of objects for the specified short device name.
    The dictionary is: {
        PCIDevice                   : CIMInstance(...),
        IoMemoryPort                : CIMInstance(...),
        SoftwareIdentity            : CIMInstance(...),
        SoftwareInstallationService : CIMInstance(...)
    }
    Return a 4-entry dictionary or None if that device doesn't exist.
    """
    devdict = {}
    searchid = "Fusion-io:{0}:pci".format(sdevice)
    devlist = conn.EnumerateInstances('FIO_PCIDevice')
    for dev in devlist:
        if dev.properties['InstanceID'].value == searchid:
            devdict['PCIDevice'] = dev
            assocs = conn.Associators(dev.path)
            for assoc in assocs:
                if assoc.classname == 'FIO_SoftwareIdentity':
                    devdict['SoftwareIdentity'] = assoc
                elif assoc.classname == 'FIO_SoftwareInstallationService':
                    devdict['SoftwareInstallationService'] = assoc
                elif assoc.classname == 'FIO_IoMemoryPort':
                    devdict['IoMemoryPort'] = assoc
    if len(devdict) < 4:
        devdict = None
    return devdict

def display_current_device_software_version(devdict, device):
    """Display the current version number for the specified device.
    HACK-O-MATIC: Older versions of the CIM provider have a bug wherein the
    VersionString property contains only the micro-rev with a leading dot. But
    that's the only place where the proper micro-rev is located. If we see the
    VersionString has a leading dot, we build a display version from maj, min,
    rev, and VersionString. Otherwise we just use the VersionString as is.
    """
    swident = devdict['SoftwareIdentity']
    verstr = swident.properties['VersionString'].value
    if verstr[0] == '.':
        maj = swident.properties['MajorVersion'].value
        min = swident.properties['MinorVersion'].value
        rev = swident.properties['RevisionNumber'].value
        bld = swident.properties['VersionString'].value[1:]
        verstr = '%d.%d.%d/%05d' % (maj, min, rev, bld)
    print "The current software version for device '%s' is %s." % (device, verstr)

def safe_convert_to_int(x):
    """Convert string to int; convert non-integer strings to -1"""
    try: return int(x)
    except ValueError: return -1

def get_status_string(conn, mclass_name, method_name, status_value):
    """Convert a method return value to a string via the class method's value map.

    Convert a method status value to its corresponding string value as specified
    by the class qualifier value map in the method from which the status was
    received. If the class doesn't have a value map for the target method,
    just use the value directly (converted to a string).

    Arguments:
      conn -- the connection from which to retrieve method class info
      mclass_name -- the name of the CIMClass of the target method
      method_name -- the method name from which to extract return values
      status_value -- the value for which to look up a string

    Return a string representation of the specified status_value.
    """
    mclass = conn.GetClass(mclass_name)
    method = mclass.methods[method_name]
    if 'ValueMap' in method.qualifiers:

        # create mapping from value to string
        value_map = dict(zip(map(safe_convert_to_int,
                method.qualifiers['ValueMap'].value),
                method.qualifiers['Values'].value))

        # return status string from status_value via value_map
        return value_map[status_value]

    return str(method.qualifiers['Value'].value)

def update_directory(conn, iservice, directory):
    """Update the firmware file store directory """
    status = conn.InvokeMethod('UpdateFirmwareDirectory', iservice.path, FirmwareDir=directory)
    return status

def upgrade_device_software(conn, devdict, ffffile, force):
    """Upgrade specified device with specified version on specified connection.

    Arguments:
      conn -- the connection on which to perform the operation
      devdict -- the device instance dictionary referencing the sis and target
      ffffile -- the fff file name to use as the update archive URI
      force -- boolean: True if we should force the operation

    Return a completion object that can be used to query the operation or None.
    """
    si = devdict['SoftwareIdentity']
    sis = devdict['SoftwareInstallationService']
    job = pywbem.cim_obj.CIMInstance('CIM_ConcreteJob')
    options = []
    if force:
        options.append(pywbem.Uint16(3))
    # InstallOptions: 3 = force-install, 4 = install
    status = conn.InvokeMethod('InstallFromURI', sis.path, Job=job,
            URI=ffffile, Target=si.path, InstallOptions=options)
    if status[0] == st_Started:
        job = status[1]['Job']
        return job
    print "Error: Unable to install desired software: %s (%d)." % (
            get_status_string(conn, 'FIO_SoftwareInstallationService',
                    'InstallFromURI', status[0]), status[0])

def job_percent_complete(conn, job):
    """Query and return the current percent-complete value from a job."""
    status = conn.GetInstance(job)
    return status.properties['PercentComplete'].value

def get_job_state(conn, job):
    """ Query a job for the JobState value """
    state = conn.GetInstance(job)
    return state.properties['JobState'].value

def job_state_string(conn, status):
    """ Return the string value for JobState """
    job_class = conn.GetClass('CIM_ConcreteJob')
    state = job_class.properties['JobState']
    # create mapping from value to string
    value_map = dict(zip(map(safe_convert_to_int,
            state.qualifiers['ValueMap'].value),
            state.qualifiers['Values'].value))

    # return status string from status via value_map
    return value_map[status]

def get_job_result(conn, job):
    """ Return the ErrorCode and ErrorDescription fields set on completion"""
    status = conn.GetInstance(job)
    result = []
    result.append(status.properties['ErrorDescription'].value)
    result.append(status.properties['ErrorCode'].value)
    return result

def display_completion_status(conn, job, timeout):
    """Display a status bar based on percent complete in specified job.
    Returns false if job times out before completion, true otherwise
    """
    state = get_job_state(conn, job)
    print "Firmware Update Job %s[%d]" % (job_state_string(conn, state), state)
    retval = True
    if state == job_Starting or state == job_Running:
        print "Updating: [",
        lastpc = 0
        pc = job_percent_complete(conn, job)
        while pc < 100:
            if not timeout > 0:
                retval = False
                break
            if pc >= lastpc:
                print " %02d" % lastpc,
                sys.stdout.flush()
                lastpc += 10
            if timeout % 5 == 0:
                print ".",
                sys.stdout.flush()
            time.sleep(1)
            timeout -= 1
            pc = job_percent_complete(conn, job)
        print "]"
    else:
        print "Job not running: %s[%d]" % (job_state_string(conn, state), state)
    return retval

def iomp_state(conn, iname):
    """Return the state of the last requested operation."""
    instance = conn.GetInstance(iname)
    return instance.properties['State'].value

# ---------------------------------------------------------------------------
# START OF MAIN

# Setup argparse data structures and parse command-line arguments

description = '''Fusion-io %(prog)s utility (3.1.4.111 tahoe@a2a30d5308e5M).
Copyright 2006-2011 Fusion-io, Inc. All rights reserved.'''

epilog = '''Summary: %(prog)s is a tool for updating ioDrive firmware.
%(prog)s can be executed in two modes: interactive and non-interactive.

Use the --interactive (-i) option to execute in interactive mode. An enumerated
list of available firmware upgrades will be displayed and the user is prompted
to choose one.

In non-interactive mode, supply a target update version either as a version tag
(x.y.z/build) or a .fff file name (something.fff) with the --use (-u) option.
The remote directory to search can be set with the --directory (-d) command.
Use the --list (-l) option to display a list of available versions and use the
--show (-s) option to display the currently installed software version.

If the version specified is less than the one currently installed, use
the --force (-f) flag to force the update request to be honored.
'''

parser = argparse.ArgumentParser(prog='fio-update-iodrive', description=description, formatter_class=argparse.RawDescriptionHelpFormatter, epilog=epilog)
parser.add_argument('device', metavar='<device>', nargs=1, help='a pathname to the device (/dev/fctX)');
parser.add_argument('-u', '--use', metavar='<version/fff-file>', nargs=1, help='specify the version and/or .fff file to use')
parser.add_argument('-i', '--interactive', action='store_true', help='display a list and prompt for a choice')
parser.add_argument('-d', '--directory', metavar='<dirname>', help='update the directory for the remote firmware file store')
parser.add_argument('-f', '--force', action='store_true', help='force upgrade against system advice')
parser.add_argument('-l', '--list', action='store_true', help='just list the versions available and exit')
parser.add_argument('-s', '--show', action='store_true', help='display the current device software version and exit')
parser.add_argument('-p', '--pretend', action='store_true', help='show what would happen but don\'t do it')
parser.add_argument('-n', '--noprompt', action='store_true', help='don\'t confirm before comitting')
parser.add_argument('-q', '--quiet', action='store_true', help='don\'t display progress')
parser.add_argument('-v', '--version', action='version', version='%(prog)s version 3.1.4.111 tahoe@a2a30d5308e5M', help="print version and exit")

wbemgroup = parser.add_argument_group('remote options')
wbemgroup.add_argument('--timeout', metavar='<seconds>', default='1800', dest='timeout', type=int, help='seconds to wait for request to complete (default:1800(30 minutes); a timeout value of 0 means don\'t wait)')
wbemgroup.add_argument('--server', metavar='<address>', dest='server', required=True, help='remote host address/dns name')
wbemgroup.add_argument('--namespace', metavar='<namespace>', dest='namespace', default='fio/cimv2', help='CIM namespace (default:fio/cimv2)')
wbemgroup.add_argument('--username', metavar='<username>', dest='username', default='anonymous', help='remote user (default:anonymous)')
wbemgroup.add_argument('--password', metavar='<password>', dest='password', default='', help="remote user's password (default:none)")
wbemgroup.add_argument('--no-ssl', default=True, action='store_false', dest='secure_flag', help='disable secure http on connection');

args = parser.parse_args()

# build a URL and credentials tuple from command line arguments

url = make_wbem_url(args)
creds = (args.username, args.password) if args.secure_flag else None

# establish connection from the URL

conn = pywbem.WBEMConnection(url, creds, default_namespace=args.namespace);

short_device = string.split(args.device[0], '/')[-1]

# get installation service for the specified device

iservice = get_installation_service(conn, short_device)
if iservice == None:
    print "Device %s not found." % short_device
    exit(0)
elif len(get_version_list(iservice)) == 0 and args.directory == None:
    print "There are no updates available for installation."
    print "Current firmware directory: %s" % get_directory(iservice)
    exit(0)

# if they only asked for a list, display it and terminate

if args.list:
    vlist = get_version_list(iservice)
    print "Firmware Directory: %s" % get_directory(iservice)
    display_version_list(vlist)
    exit(0)

# check for option clashes (-q, -i, -d)

if args.interactive:
    if args.quiet:
        print "Error: Unable to use interactive (-i) mode quietly (-q)."
        exit(1)
    if args.directory:
        print "Error: Unable to update directory (-d) in interactive mode (-i)."
        exit(1)

# parse path name (/dev/fctx); locate correct object instances

devdict = build_device_dictionary(conn, short_device)
if devdict == None:
    print "Error: Unable to locate device '%s'." % short_device
    exit(1)

# if not interactive, ensure a version and/or directory is specified

if not args.interactive:
    if args.directory != None:
        # Specified a new firmware directory, update it before continuing
        print "Updating firmware directory to %s" % args.directory
        if not update_directory(conn, iservice, args.directory):
            print "Unable to update firmware directory."
            exit(1)
        else:
            print "Firmware directory updated to %s." % args.directory
            if args.use == None:
                exit(0)

    if args.use == None:
        print "Error: Please specify a version or .fff file to use, or else use interactive mode."
        exit(1)
    else:
        version = get_specific_version(iservice, args.use[0])
        if args.directory != None:
            # Wait for available versions list to populate after changing directory (max 20 seconds)
            print "Updated directory, waiting for updated version list...",
            timeout = 20
            while timeout > 0 and version == None:
                time.sleep(5)
                timeout -= 5
                print ".",
                sys.stdout.flush()
                iservice = get_installation_service(conn, short_device)
                version = get_specific_version(iservice, args.use[0])
            print "Done"
        if version == None:
            print "Error: Desired version '%s' is not available." % args.use
            exit(1)

# tell 'em what they currently have so they can make an intelligent choice

if not args.quiet:
    display_current_device_software_version(devdict, args.device)
    if args.show:
        exit(0)

# if interactive let them choose a version

if args.interactive:
    version = prompt_for_version(iservice)
    if version == None:
        exit(0)

# if prompt desired, prompt for permission

if not args.noprompt:
    answer = raw_input("Update device '%s' to software version %s from file %s? [y/N] " \
            % (args.device, version[0], version[1]))
    if len(answer) == 0 or (answer[0] != 'Y' and answer[0] != 'y'):
        print "Aborting."
        exit(1)

# Verify device is not attached before beginning

if iomp_state(conn, devdict['IoMemoryPort'].path) == iomp_Attached:
    print "Device %s is attached.  You must detach the device before updating" % short_device
    exit(1)

# permission granted - do it...

if not args.quiet:
    print "Updating device '%s' to software version %s from file %s." \
            % (args.device, version[0], version[1])

# ...unless we've been told to fake it, that is

if not args.pretend:
    job = upgrade_device_software(conn, devdict, version[1], args.force)
    if job != None and not args.quiet:
        if display_completion_status(conn, job, args.timeout):
            print "Done"
            result = get_job_result(conn, job)
            if result[1] == 0:
                print "Firmware update successful, reboot machine to activate new firmware"
            else:
                print "Firmware update failed: %s(%s)" % (result[0], result[1])
        else:
            print "Timeout reached, Update operation still running"
elif not args.quiet:
    print "(Pretending)"
    print "Done"

# END OF MAIN
# ---------------------------------------------------------------------------
