#!/usr/bin/env python
#
# fio_claimlib.py     ESX/ESXi claim/unclaim commands for attach/detach operations
#

import sys
import subprocess
import string
import re
import os
import time
import pywbem

__author__ = "Fusion-io management software team"
__date__ = "$Jan. 26, 2012 11:00:00 AM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

# Global variables set by init()

commands_ = {}
hbas_ = []
mpxs_ = []
init_flag_ = False

# Initializes the fio_claimlib library by retrieving all device data
# from the target system and building the command list
# Called from unclaimed(), unclaim(), claim()
def init(args):
    # Check if an executable exists in the system PATH
    def which(program):
        def is_exe(fpath):
            return os.path.exists(fpath) and os.access(fpath, os.X_OK)

        fpath, fname = os.path.split(program)
        if fpath:
            if is_exe(program):
                return program
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                exe_file = os.path.join(path, program)
                if is_exe(exe_file):
                    return exe_file
        return None

    # Declare that we are modifying the global copies of these variables
    global commands_, hbas_, mpxs_, init_flag_

    # Check if we already initialized
    if init_flag_:
        return

    # Make sure the vSphere CLI tools are available before continuing
    if which("esxcli") is None:
        print "This script requires VMware vSphere CLI tools"
        print "Please ensure the vSphere CLI tools are installed and available in the system PATH"
        exit(1)

    # Build command list
    commands_ = get_claim_commands(args)

    # Build device lists
    hbas_ = get_all_hbas(args)
    mpxs_ = get_all_mpxs(args)

    init_flag_ = True

# Returns a dictionary of the necessary claim/unclaim commands by
# checking the ESX version on the host system
def get_claim_commands(args):
    # Get ESX version from the base CIM provider
    url = "{0}://{1}".format('https' if args.secure_flag else 'http', args.server)
    creds = (args.username, args.password) if args.secure_flag else None
    conn = pywbem.WBEMConnection(url, creds, default_namespace="root/cimv2")

    sysnames = conn.EnumerateInstanceNames('VMware_HypervisorSoftwareIdentity')

    unclaim = unmount = filesys = claim = None

    for sys in sysnames:
        if "ESXi 5" in sys.keybindings['InstanceID']:
            unclaim = ['storage', 'core', 'claiming', 'unclaim', '-t', 'device', '-d']
            unmount = ['storage', 'filesystem', 'unmount', '-n', '-u']
            filesys = ['storage', 'filesystem', 'list']
            claim = ['storage', 'core', 'claimrule', 'run']
            break
        elif "ESXi 4" in sys.keybindings['InstanceID']:
            unclaim = ['corestorage', 'claiming', 'unclaim', '-t', 'device', '-d']
            claim = ['corestorage', 'claimrule', 'run']
            break
        elif "ESX 4" in sys.keybindings['InstanceID']:
            unclaim = ['corestorage', 'claiming', 'unclaim', '-t', 'device', '-d']
            claim = ['corestorage', 'claimrule', 'run']
            break
        else:
            print "Unsupported ESX version: %s" % sys.keybindings['InstanceID'].value
            print "Supported versions are ESX 4.x, ESXi 4.x, ESXi 5.0"
            exit(1)

    commands = {}
    commands['unclaim'] = unclaim
    commands['unmount'] = unmount
    commands['filesys'] = filesys
    commands['claim'] = claim

    return commands

# Get all hba device names on the system
# Returns list of : <hba name> <driver> <UID> <PCI addr> <Vendor/model>
def get_all_hbas(args):
    return get_all_devices(args, '-a')

# Get all mpx device names on the system
# Returns list of : <hba name> <mpx name>
def get_all_mpxs(args):
    return get_all_devices(args, '-A')

# Get all filesystem uuids on the system
# returns list of : <mpx name> <device path> <filesystem uuid>....
def get_all_uuids(args):
    return get_all_devices(args, '-m')

# Use the esxcfg-scsidevs command to get all devices
# of a given type on the host system
# flag: '-a' for hba, '-A' for mpx, or '-m' for uuid
def get_all_devices(args, flag):
    # On windows we have to include .pl, on other platforms the script is executable directly
    if "win" in sys.platform:
        cmd_name = 'esxcfg-scsidevs.pl'
    else:
        cmd_name = 'esxcfg-scsidevs'
    cmd = [cmd_name, flag, '--username', args.username, '--password', args.password, \
            '--server', args.server]

    p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
    p.wait()
    lines = p.stdout.readlines()

    output = []
    for line in lines:
        output.append(line.rstrip())
    return output

# Get all filesystems on the host system and their mounted state
# Returns list of <mount point> <volume name> <uuid> <mounted state>....
def get_all_filesystems(args):
    if commands_['filesys'] is not None:
        cmd = ['esxcli', '--username', args.username, '--password', args.password, \
              '--server', args.server]
        for item in commands_['filesys']:
            cmd.append(item)
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        p.wait()
        lines = p.stdout.readlines()

        output = []
        for line in lines:
            output.append(line.rstrip())
        return output
    else:
        return None

# Get hba string from device node name
# Eg: /dev/fct0 -> vmhba2
def dev_to_hba(hbas, dev):
    fct = re.findall("[0-9]+", dev)[0]
    search = "c{0}d{0}".format(fct)

    for hba in hbas:
        if "block.fio" in hba and search in hba:
            return hba.split()[0]
    return None

# Get device node from hba string
# Eg: vmhba2 -> /dev/fct0
def hba_to_dev(hbas, hba):
    for line in hbas:
        if line.split()[0] == hba:
            dev = line.split()[2]
            # dev = block.fio/cXdX:0
            fct = re.findall("[0-9]+", dev)[0]
            return "/dev/fct%s" % fct
    return None

# Get mpx device string from hba
# Eg: vmhba2 -> mpx.vmhba2:C0:T0:L0
def hba_to_mpx(mpxs, hba):
    for mpx in mpxs:
        if mpx.split()[0] == hba:
            # Have to check for validity of mpx name because vCLI tools
            # return a placeholder for an hba with no mpx
            mpx = mpx.split()[-1]
            if "mpx" in mpx:
                return mpx
    return None

# Get uuids of all filesystems using mpx device
# Eg: mpx.vmhba2:C0:T0:L0 -> xxxx-xxxx-xxxxxxxx-xxxx
def mpx_to_uuids(uuids, mpx):
    mpx_uuids = []
    for uuid in uuids:
        if uuid.split()[0] == mpx:
            mpx_uuid = uuid.split()[2]
            mpx_uuids.append(mpx_uuid)
    return mpx_uuids

# Check if the filesystem represented by a given uuid is currently mounted
def is_mounted(filesystems, uuid):
    for filesystem in filesystems:
        if filesystem.split()[2] == uuid and filesystem.split()[3] == "true":
            return True
    return False

# Check for unclaimed devices, return the name of any device other than
# the specified device that is found in unclaimed state.  "unclaimed"
# means a device that has an hba but is missing an mpx
def unclaimed(args):
    device = args.device[0]

    # Make sure claim lib has been initialized
    init(args)

    # Find all hba's representing a fusion-io device
    fio_hbas = []
    for hba in hbas_:
        if "block.fio" in hba:
            fio_hbas.append(hba.split()[0])

    # Check if any devices are unclaimed
    for hba in fio_hbas:
        mpx = hba_to_mpx(mpxs_, hba)
        if mpx is None:
            dev = hba_to_dev(hbas_, hba)
            if dev is not None and dev != device:
                return dev
    return None

# Unclaim a device for safe detaching
def unclaim(args):
    cmd_base = ['esxcli', '--username', args.username, '--password', args.password, \
               '--server', args.server]

    device = args.device[0]

    # Make sure claim lib has been initialized
    init(args)

    # Get the hba for the device
    hba = dev_to_hba(hbas_, device)
    if hba is None:
        # In this situation, there is no hba, therefore there is nothing
        # to unclaim, so a normal detach can be done safely
        return 0

    # Get the mpx for the hba
    mpx = hba_to_mpx(mpxs_, hba)
    if mpx is None:
        if not args.quiet_flag:
            print "%s (%s) already unclaimed" % (device, hba)
        return 0

    # Get uuid and filesystem lists in order to unmount
    uuids = get_all_uuids(args)
    filesystems = get_all_filesystems(args)

    # Get all uuids related to the mpx and unmount if possible
    dev_uuids = mpx_to_uuids(uuids, mpx)
    for uuid in dev_uuids:
        if commands_['unmount'] is not None:
            cmd = cmd_base
            for item in commands['unmount']:
                cmd.append(item)
            cmd.append(uuid)
            if is_mounted(filesystems, uuid):
                if not args.quiet_flag:
                    print "Unmounting VMFS %s" % uuid
                p = subprocess.Popen(cmd)
                result = p.wait()
                if result != 0:
                    print "Unable to unmount VMFS %s" % uuid
                    return 1
            elif not args.quiet_flag:
                    print "VMFS %s already unmounted" % uuid

    cmd = cmd_base
    for item in commands_['unclaim']:
        cmd.append(item)
    cmd.append(mpx)
    if not args.quiet_flag:
        print "Unclaiming '%s' (%s)..." % (mpx, device),
        sys.stdout.flush()

    # Unclaim the mpx
    retries = 10
    while retries > 0:
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        result = p.wait()
        if result == 0:
            if not args.quiet_flag:
                print "Done"
            break
        else:
            if not args.quiet_flag:
                print ".",
                sys.stdout.flush()
            time.sleep(1)
            retries -= 1

    if retries == 0:
        # If retry count hit 0, we failed to unclaim the device
        print "Unable to unclaim '%s'" % mpx
        return 1
    else:
        # If unclaim succeeded, we need to wait for the vCLI tools to detect the
        # now missing mpx, which isn't always picked up immediately, in order for
        # the next attach/detach command to be safe
        retries = 10
        while retries > 0:
            new_mpxs = get_all_mpxs(args)
            mpx = hba_to_mpx(new_mpxs, hba)
            if mpx is None:
                break
            else:
                time.sleep(1)
                retries -= 1
        return 0

# Claim a device after attaching
def claim(args):
    cmd = ['esxcli', '--username', args.username, '--password', args.password, \
           '--server', args.server]

    device = args.device[0]

    # Make sure claim lib has been initialized
    init(args)

    for item in commands_['claim']:
        cmd.append(item)
    p = subprocess.Popen(cmd)
    p.wait()

    # If this is the first attach (i.e. was booted with auto_attach=0) we must wait
    # for an hba to be available before we can check the mpx availability
    hba = dev_to_hba(hbas_, device)
    if hba is None:
        retries = 10
        while retries > 0:
            new_hbas = get_all_hbas(args)
            hba = dev_to_hba(new_hbas, device)
            if hba is not None:
                break
            else:
                time.sleep(1)
                retries -= 1

    # Wait for the vCLI tools to detect the mpx that was just added, so that the next 
    # attach/detach command does not block thinking there is an unclaimed device
    # if we never found a new hba, return immediately
    if hba is not None:
        retries = 10
        while retries > 0:
            new_mpxs = get_all_mpxs(args)
            mpx = hba_to_mpx(new_mpxs, hba)
            if mpx is not None:
                break
            else:
                time.sleep(1)
                retries -= 1
