/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_COMMON_TYPES_H_
#define _INCLUDE_FIO_PUBLIC_COMMON_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif


#if defined(WINNT) || defined(WIN32)
#include <windows.h>
typedef UINT8   uint8_t;
typedef UINT16  uint16_t;
typedef UINT32  uint32_t;
typedef UINT64  uint64_t;
typedef INT8    int8_t;
typedef INT16   int16_t;
typedef INT32   int32_t;
typedef INT64   int64_t;
#else
#include <stdint.h>
#include <stdbool.h>
#endif

#include <time.h>

#ifdef DllExport
# undef DllExport
#endif

#define MAX_DEVICE_PATH         260         /**< The max length that a path to a device can be, eg: /dev/fct0. */

#if defined(WIN32) || defined(WINNT)
#define DllExport   __declspec( dllexport )
#else
#define DllExport
#endif

/** \brief Results of a data item lookup on a device. */
typedef enum
{
    FIO_STATUS_SUCCESS              = 0,    /**< Success */
    FIO_STATUS_DEVICE_NOT_FOUND     = 1,    /**< Couldn't find the specified device */
    FIO_STATUS_FAILURE              = 2,    /**< Driver couldn't get the value */
    FIO_STATUS_FAILURE_PERM         = 3,    /**< User did not have permission to get the value */
    FIO_STATUS_NOT_IMPLEMENTED      = 4,    /**< Reserved for internal use. */
    FIO_STATUS_ENOMEM               = 5,    /**< Out of memory while executing SDK function. */
    FIO_STATUS_PCI_UNSUPPORTED      = 6,    /**< Operation using PCI selector modes not supported on the current platform. */
    FIO_STATUS_DRIVER_LOADED        = 7,    /**< Unsupported operation when the driver is loaded. */
    FIO_STATUS_INVALID_INPUT        = 8,    /**< Invalid input parameters. */
} fio_status_result_t;

/**
 * \brief Enumeration of iteration modes. Used internally by iteration functions.
 */
typedef enum
{
    FIO_ITER_NAME         = 0,          /**< Iterate by device/file/key name. */
    FIO_ITER_INDEX        = 1,          /**< Iterate by index or position in an array or list. */
    FIO_ITER_HANDLE       = 2,          /**< DEPRECATED. Iterate by handle or selector. */
    FIO_ITER_PCI          = 3,          /**< Iterate by PCI address (domain, bus, device, and function.). */
    FIO_ITER_PCI_STRING   = 4,          /**< Iterate by string version of PCI address (eg 0:6:0.0) */
} fio_iterator_mode_t;

/**
 * \brief Object used to iterate over various objects.
 *      The structure should be initialized by FIO_STRUCT_INIT before use.
 *      All other fields of the structure are used internally by iterator functions and should not be set by the caller.
 */
typedef struct fio_iterator
{
    uint32_t            lib_version;    /**< Fusion SDK API version (from FIO_STRUCT_INIT) */
    fio_status_result_t result;         /**< [out] Current state of the iterator.
                                                FIO_STATUS_SUCCESS indicates a device/object was found.
                                                FIO_STATUS_DEVICE_NOT_FOUND indicates the end of the list has been reached.
                                                All other values indicate an error. */
    fio_iterator_mode_t mode;           /**< INTERNAL: The mode being used to iterate over the devices. */
    void*               position;       /**< INTERNAL: The current position of the iterator. */
} fio_iterator_t;

/**
 * \brief Callback function for receiving progress updates.
 * \param id identifier passed in when calling original operation.
 * \param percent percent complete ranging from 0 to 100.
 * \param context user context data.
 */
typedef void(*fio_cb_fn_t)(uint64_t id, int16_t percent, void* context);

/**
 * \brief structure used to register for callbacks from the SDK.  All operations will block until they
 *  are complete, so the only way to tell how far along they are is to register with callbacks via
 *  this structure */
typedef struct fio_progress_callback2
{
    uint32_t        lib_version;        /**< Fusion -io SDK API version (populated by FIO_STRUCT_INIT) */
    uint64_t        id;                 /**< Unique ID passed to the callback function so it can identify the purpose
                                           of the callback operation */
    fio_cb_fn_t     callback;           /**< Function to call with the progress updates */
    clock_t         callback_period_ms; /**< Number of milliseconds to wait between each callback */
    void*           context;            /**< Additional user context to propagate. This is passed to the callback
                                           function when it is invoked. */
} fio_progress_callback2_t;

/** \brief Enumeration of compatibility levels a slot and card may have. */
typedef enum
{
    COMPATIBILITY_INCOMPATIBLE  = 0x000,    /**< Insufficient to run. */
    COMPATIBILITY_SUBOPTIMAL    = 0x010,    /**< Sufficient to run, but with possibly degraded performance. */
    COMPATIBILITY_OPTIMAL       = 0x800,    /**< The slot is providing sufficient for maximum performance. */

    COMPATIBILITY_UNKNOWN       = 0x8000    /**< Compatibility is unknown, due to missing information. */
} fio_iodimm_compatibility_t;

/** \brief Possible speeds that a PCIe link may have, in GT(T=transfers) / second. */
typedef enum
{
    FIO_PCIE_LINKSPEED_UNAVAIL = 0,       /**< PCIe link speed is unavailable */
    FIO_PCIE_LINKSPEED_2_5     = 1,       /**< PCIe link speed is 2.5 GT/sec per lane */
    FIO_PCIE_LINKSPEED_5_0     = 2,       /**< PCIe link speed is 5.0 GT/sec per lane */
    FIO_PCIE_LINKSPEED_8_0     = 3,       /**< PCIe link speed is 8.0 GT/sec per lane */
} fio_pcie_linkspeed_t;

/** \brief Struct containing 'static' PCIexpress info, that should not change during operation. */
typedef struct fio_pciexpress_info
{
    uint8_t              is_valid;              /**< non-zero: the following pcie information is present and valid. */
    uint8_t              linkwidth;             /**< Current number of PCIe lanes negotiated. 0 if not available. */
    fio_pcie_linkspeed_t linkspeed;             /**< Current negotiated linkspeed value. 0 if not available.
                                                     Note that this link speed is measured in transfers per sec,
                                                     and for every 8 logical bits transferred over PCIe, 2 control
                                                     bits are transferred.  */
    float                bandwidth_mbytes_sec;  /**< Total bandwidth of the PCIe link (linkwidth * linkspeed), MBytes/sec.
                                                     This is measured in transfers per second; see linkspeed field doc
                                                     for more details. */
    float                slot_power_watts;      /**< Power available from the slot into which this adapter or ioDimm is inserted. */
} fio_pciexpress_info_t;

/** \brief "Static" PCI information for the device */
typedef struct fio_pci_info
{
    uint32_t lib_version;               /**< Fusion -io SDK API version (populated by FIO_STRUCT_INIT) */
    uint8_t  bus;                       /**< PCI bus */
    uint8_t  device;                    /**< PCI device */
    uint8_t  function;                  /**< PCI function */
    uint32_t vendor_id;                 /**< PCI vendor ID */
    uint32_t device_id;                 /**< PCI device ID */
    uint8_t  slot_number;               /**< PCI slot number where the device is installed.  The slot number is not
                                               available on all motherboards */
    uint8_t  is_slot_number_valid;      /**< Indicates whether the slot number is valid (non-zero means valid).
                                               The slot number is not available on all motherboards. */
    uint16_t subsystem_vendor_id;       /**< PCI subsystem vendor ID. If this is an OEM device, it will contain an OEM vendor ID. */
    uint16_t subsystem_device_id;       /**< PCI subsystem device ID. If If this is an OEM device, it will contain an OEM device ID. */

    /* API version 2 (FIO_SDK_API_VERSION_2) and later: */
    fio_pciexpress_info_t       pcie;                   /**< PCIe non-changing info. */
    fio_iodimm_compatibility_t  bandwidth_compatibility;/**< PCIe link bw compatibility of this adapter in the motherboard slot */
    fio_iodimm_compatibility_t  power_compatibility;    /**< Wattage compatibility of this adapter in the motherboard slot */
} fio_pci_info_t;

/** \brief PCIe dynamic information that may change during operation. */
typedef struct fio_iodimm_dynamic_pci_info
{
    uint32_t lib_version;               /**< Fusion -io SDK API version (populated by FIO_STRUCT_INIT) */

    uint16_t errors;                    /**< Flag that indicates pcie errors. Zero: no errors, non-zero: errors, check below. */
    uint8_t  error_correctable;         /**< bool indicating a correctable pcie error occurred since last cleared. */
    uint8_t  error_nonfatal;            /**< bool indicating a non-fatal pcie error occurred since last cleared. */
    uint8_t  error_fatal;               /**< bool indicating a fatal pcie error occurred since last cleared. */
    uint8_t  error_unrecognized_req;    /**< bool indicating an unrecognized request pcie error occurred since last cleared. */
} fio_iodimm_dynamic_pci_info_t;

/** \brief Generic version information struct */
typedef struct fio_version
{
    uint32_t major;                     /**< Major version number. (x in x.y.z) */
    uint32_t minor;                     /**< Minor version number. (y in x.y.z) */
    uint32_t micro;                     /**< Micro version number. (z in x.y.z) */
    uint32_t nano;                      /**< Nano version, or build number */
} fio_version_t;

/** \brief Firmware version */
typedef struct fio_firmware_version
{
    uint32_t revision;      /**< Firmware's revision number (legacy versioning scheme) */
    uint32_t major;         /**< Major version of the firmware (x in x.y.z) */
    uint32_t minor;         /**< Minor version of the firmware (y in x.y.z) */
    uint32_t micro;         /**< Micro version of the firmware (z in x.y.z) */
} fio_firmware_version_t;

/** \brief Firmware version information */
typedef struct fio_firmware_version_info
{
    uint32_t lib_version;               /**< Fusion-io SDK API version (populated by FIO_STRUCT_INIT) */
    fio_firmware_version_t current_ver; /**< Version of the currently installed firmware.*/
    fio_firmware_version_t minimum_ver; /**< Minimum version of firmware supported by this driver */
    fio_firmware_version_t maximum_ver; /**< First version of firmware NOT supported by this driver. NB revision is not valid. */
} fio_firmware_version_info_t;

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_COMMON_TYPES_H_ */
