/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_IODIMM_MGMT_H_
#define _INCLUDE_FIO_PUBLIC_IODIMM_MGMT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/iodimm_types.h>

/**
 * \brief Get the adapter (baseboard / carrier) information from an ioDimm.  This data is static and will not change if
 *  called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *      If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param adapter_info Pointer to pre-allocated structure where the adapter information is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_adapter_info(fio_iodimm_selector_t* input,
                            fio_iodimm_adapter_info_t* adapter_info);

/**
 * \brief Get the adapter (baseboard / carrier) dynamic information from an ioDimm.  This data is dynamic and may change if
 *          called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *      If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param adapter_dynamic Pointer to pre-allocated structure where the dynamic adapter information is copied.
 *      This value is guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_adapter_dynamic_pci_info(fio_iodimm_selector_t* input, fio_iodimm_dynamic_pci_info_t* adapter_dynamic);

/**
 * \brief Get product information from an ioDimm. This data is static and will not change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *      If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param product_info Pointer to a pre-allocated structure where the product information is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_product_info(fio_iodimm_selector_t* input,
                            fio_iodimm_product_info_t* product_info);

/**
 * \brief Get PCI information from an ioDimm.  This data is static and will not change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param pci_info Pointer to a pre-allocated structure where the PCI information is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_pci_info(fio_iodimm_selector_t* input, fio_pci_info_t* pci_info);

/**
 * \brief Get PCIe errors information from an ioDimm.  This data is dynamic and may change if called on a periodic basis.
 *
 * \param input: Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param iodimm_dynamic: Pointer to a pre-allocated structure where the dynamic PCI information is copied.
 *      This value is guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 * \note There may be initial PCIe errors returned after boot-up. 
 * \note Returned PCIe errors are persistent unless cleared manually (ie eg, using lspci or fio-pci-check)
 *  
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_dynamic_pci_info(fio_iodimm_selector_t* input, fio_iodimm_dynamic_pci_info_t* iodimm_dynamic);

/**
 * \brief Get the device identifiers from an ioDimm. This data is dynamic and may change between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param identifiers Pointer to a pre-allocated structure where the identifiers are copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_device_identifiers(fio_iodimm_selector_t* input,
                                  fio_iodimm_device_identifiers_t* identifiers);

/**
 * \brief Get the current state from an ioDimm. This data is dynamic and may change between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param state Pointer to a pre-allocated structure where the state information is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_device_state(fio_iodimm_selector_t* input,
                            fio_iodimm_state_info_t* state);

/**
 * \brief Get the version of the driver. This data is dynamic and may change between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param ver_info Pointer to a pre-allocated structure where the version info is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_driver_version(fio_iodimm_selector_t* input,
                              fio_iodimm_driver_ver_info_t* ver_info);

/**
 * \brief Determine whether this library is compatible with the current driver.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param compatible Pre-allocated unsigned char where the result is stored. If the library is compatible,
 *      this value is set to 1; otherwise it is set to 0.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_driver_compatibility(fio_iodimm_selector_t* input,
                                    uint8_t* compatible);

/**
 * \brief Get the static format information from an ioDimm.  This data is static and will not change if
 *  called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function  internally opens and closes it.
 * \param format_info Pointer to a pre-allocated structure where the format information is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_static_format_info(fio_iodimm_selector_t* input,
                                  fio_iodimm_static_format_info_t* format_info);

/**
 * \brief Get the dynamic format information from an ioDimm.  This data is dynamic and may change
 *  between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param format_info Pointer to a pre-allocated structure where the format information is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_dynamic_format_info(fio_iodimm_selector_t* input,
                                   fio_iodimm_dynamic_format_info_t* format_info);

/**
 * \brief Calculate potential formatting sizes for this ioDimm.
 * Calculates format sizes bases on a given sector size. 
 * The larger the sector size, the larger the max format size of the device.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param format_sizes Pointer to a pre-allocated structure where the format calculations are returned. This value is
 *      only guaranteed to be populated if FIO_STATUS_SUCCESS is returned.
 * \note It is possible to specify a format size which is larger than the physical capacity of the ioDrive. 
 *         This capability is for special file systems and add-ons and may not work for standard use.
 * \note Format sizes other than the default have positive and negative endurance and performance implications. 
 *         Sizes greater than the default negatively impact longevity and performance.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 *         A return value of FIO_STATUS_INVALID input may indicate:
 *          an invalid sector size specified in the format_sizes struct,
 *          or FIO_STRUCT_INIT() was not called on the two input structs, or else they are NULL.
 */
DllExport
fio_status_result_t
fio_iodimm_get_format_sizes(fio_iodimm_selector_t* input, fio_iodimm_format_sizes_t* format_sizes);

/**
 * \brief Get the current status of flashback protection from an ioDimm.  This data is dynamic and may change
 *      between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param state Pointer to a pre-allocated structure where the flashback state is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_flashback_indicator(fio_iodimm_selector_t* input,
                                   fio_iodimm_flashback_t* state);

/**
 * \brief Get the write throttling status of an ioDimm. If write throttling is engaged, the reason for it is returned.
 *      This data is static and will not change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param state Pointer to a pre-allocated structure where the write throttling state is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_write_throttling(fio_iodimm_selector_t* input,
                                fio_iodimm_write_throttling_t* state);

/**
 * \brief Get the current temperature of the device in degrees C, from an ioDimm.  This data is dynamic and may change
 *      between invocations of this function.
 * \deprecated This function has been deprecated in favor of the more informational version: fio_iodimm_get_temperatures
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param temp Pointer to a pre-allocated uint8_t where the current device temperature is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_engine_temp_c(fio_iodimm_selector_t* input, int8_t* temp);

/**
 * \brief Get the current temperatures of the device from an ioDimm.  This data is dynamic and may change
 *      between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param temperatures Pointer to a pre-allocated fio_iodimm_temperatures_t where the current device temperatures are copied.
 *      These values are guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_temperatures(fio_iodimm_selector_t* input, fio_iodimm_temperatures_t* temperatures);

/**
 * \brief Return the current setpoints for the temperature alarm. When the temperature rises above the high setpoint,
 *      the driver begins to shut down. This data is static and will not change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param setpoints Pointer to a pre-allocated structure where the setpoints are copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_temperature_alarm_setpoints(fio_iodimm_selector_t* input,
                                           fio_iodimm_temperature_setpoints_t* setpoints);

/**
 * \brief Get the device usage data from an ioDimm.  This data is dynamic and may change between invocations of this
 *      function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param usage_data Pointer to pre-allocated structure where the usage data is copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_usage(fio_iodimm_selector_t* input,
                     fio_iodimm_usage_t* usage_data);

/**
 * \brief Get entries from the total capacity history table. An array of entries is returned, showing the loss of
 *      reserve capacity over time due to NAND flash failures.
 *      The total capacity is defined as the formatted size plus the internal reserve.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param table The desired number of entries is requested in this structure. This MUST point to enough storage for
 *      the returned entries. To get the number of entries needed, set table->size_entries to zero. This function
 *      then sets size_entries to the number of entries between the timestamps.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_logical_capacity_history(fio_iodimm_selector_t* input,
                                        fio_iodimm_history_table_t* table);

/**
 * \brief Get entries from the usage history table.  An array of entries is returned, showing the write volume (total 
 *      number of physical bytes written) at different points in time. This can be correlated with flash failures.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param table The desired number of entries is requested in this structure. This MUST point to enough storage for
 *      the returned entries. To get the number of entries available, set table->num_entries to zero. This function
 *      then sets num_entries accordingly.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_usage_history(fio_iodimm_selector_t* input,
                             fio_iodimm_history_table_t* table);

/**
 * \brief Get entries from the temperature history table.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param table The desired number of entries is requested in this structure. This MUST point to enough storage for
 *      the returned entries. To get the number of entries available, set table->num_entries to zero. This function
 *      then sets num_entries accordingly.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_temperature_history(fio_iodimm_selector_t* input,
                                   fio_iodimm_history_table_t* table);

/**
 * \brief Set the device beacon status of an ioDimm. If the beacon is enabled, the LEDs are all turned on. This can
 *      be used to physically identify a device in a chassis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param beacon_on Zero turns the beacon off; any other value turns it on.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_set_beacon(fio_iodimm_selector_t* input,
                      uint8_t beacon_on);

/**
 * \brief Get the device beacon status from an ioDimm.  This data is dynamic and may change between invocations of this
 *      function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param beacon_on Pointer to where the beacon status is returned. Zero means beacon is off, any other value means it is on.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_beacon(fio_iodimm_selector_t* input,
                      uint8_t* beacon_on);

/**
 * \brief Retrieves the current state and blink rate of the LEDs from an ioDimm.  This data is dynamic and may change
 *  between invocations of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param led_status Pointer to a pre-allocated structure where the LED status is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_led_status(fio_iodimm_selector_t* input,
                          fio_led_status_t* led_status);

/**
 * \brief Get the alternate information from an ioDimm. This data is static and will not change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param alt_info Pointer to a pre-allocated structure where the alternate information is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_alt_info(fio_iodimm_selector_t* input, fio_iodimm_alt_info_t* alt_info);

/**
 * \brief Get the version of firmware installed on the device, as well as the minimum supported version. Note: After
 *      updating firmware on a device, you must reboot the machine before this function can return the new firmware
 *      version.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param ver_info Pointer to where the firmware version info is returned. This value is guaranteed to be populated
 *      with the correct value if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_firmware_version_info(fio_iodimm_selector_t* input,
                                     fio_firmware_version_info_t* ver_info);
/**
 * \brief Get various system power usage data from an ioDimm. This data is dynamic and may change between invocations
 *      of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param power_usage Pointer to a pre-allocated structure where the power usage data is copied. This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_power_usage(fio_iodimm_selector_t* input,
                           fio_iodimm_power_usage_t* power_usage);

/**
 * \brief Clear the peak registers for the PCIe power monitor. These registers must be cleared if you want to
 *      observe any peaks lower than the currently captured peaks.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param clearmask Bitfield mask of the registers that will be cleared. See fio_iodimm_power_usage_peaks_t for a
 *      convenient enum of these values.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_clear_power_usage_peaks(fio_iodimm_selector_t* input,
                                   fio_iodimm_power_usage_peaks_t clearmask);

/**
 * \brief Get the current low and high setpoints of the power monitoring system. This data is static and will not
 *      change if called on a periodic basis.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param setpoints Pointer to pre-allocated structure where the setpoints are copied. This value is guaranteed to be
 *       populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_power_setpoints(fio_iodimm_selector_t* input,
                               fio_iodimm_power_setpoints_t* setpoints);

/**
 * \brief Get the current, active hardware alerts. If there are active alerts, the type field will contain the value
 *        for the priority alert. The current value and setpoints will be filled in with the respective values for
 *        that alert type. Secondary alerts, if present, will each have a bit set in the active_mask. If there are no
 *        alerts, the type and alert_mask fields will have a value of hw_alert_none.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param alerts Pointer to pre-allocated structure where the alert data is copied. This value is guaranteed to be
 *       populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_hw_alerts(fio_iodimm_selector_t* input, fio_iodimm_hw_alerts_t* alerts);

/**
 * \brief Get the current health of the ioDrive, as indicated by capacity. IMPORTANT NOTE: This is a 'heavy'
 *      function, in that it makes iterative calls to the driver and may affect performance if called too
 *      frequently. Call only as often as necessary. This data is dynamic and may change between invocations
 *      of this function.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param health Pointer to a pre-allocated structure where the health information will be copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_health(fio_iodimm_selector_t* input, fio_iodimm_health_t* health);

/**
 * \brief Get the version of the SDK library. The SDK is backwards-compatible, but this function is useful for an application
 *         to determine whether it is out of date and could benefit from being rebuilt with the new SDK header files.  Note
 *         that this is NOT the version passed in to fio_sdk_init.
 *
 * \return the SDK API version.
 */
DllExport
uint32_t
fio_get_sdk_api_version(void);

/**
 * \brief Get various ioDimm configuration settings.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param settings Pointer to a pre-allocated structure where the settings will be copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_settings(fio_iodimm_selector_t* input, fio_iodimm_settings_t* settings);

/**
 * \brief Get the amount of memory required if this iodimm will have preallocation turned on.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param info Pointer to a pre-allocated structure where the info will be copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_prealloc_info(fio_iodimm_selector_t* input, fio_iodimm_prealloc_info_t* info);

/**
 * \brief Get lifetime extender information.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param info Pointer to a pre-allocated structure where the info will be copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_lifetime_extender_state(fio_iodimm_selector_t* input, fio_iodimm_lifetime_extender_state_t* info);

/**
 * \brief Get write regulation level info.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param type Specifies the write regulation type for the level info request.
 * \param level_info Pointer to a pre-allocated structure where the info will be copied.  This value is
 *      guaranteed to be populated only if FIO_STATUS_SUCCESS is returned.
 *
 * \return See the fio_status_result_t declaration for return value descriptions.
 */
DllExport
fio_status_result_t
fio_iodimm_get_write_regulation_level_info(fio_iodimm_selector_t* input,
                                           fio_iodimm_write_regulation_level_info_t* level_info);

#ifdef __cplusplus
}
#endif

#endif  /* _INCLUDE_FIO_PUBLIC_IODIMM_MGMT_H_ */
