/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_IODIMM_OPERATIONS_H_
#define _INCLUDE_FIO_PUBLIC_IODIMM_OPERATIONS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/iodimm_types.h>

/** 
 * \brief Initialize the SDK.  This must be done before beginning to use the SDK's
 *    functionality, in order to guarantee proper behavior.
 *
 * \param api_version The SDK version being used by the client of this SDK.  Used internally when allocating 
 *      structures to assure backward compatibility.
 *
 * \return 0 on success, != 0 on failure.
 */
DllExport
int
fio_sdk_init(uint32_t api_version);

/** 
 * \brief Shutdown the SDK.  This should be done prior to exiting your application to assure
 *  that all internally opened handles are closed.
 */
DllExport
void
fio_sdk_fini(void);

/**
 * \brief Attach the ioDimm to allow it to function as a block device. This function is thread-safe and may be called
 *      on multiple ioDimm's in parallel.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request the attach request structure (optional, may be NULL).  See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param callback structure (optional, may be NULL).  See the header (common_types.h) for more information on how to populate this struct.
 * \return see fio_iodimm_attach_rsp_t documentation for details.
*/
DllExport
fio_iodimm_attach_status_t
fio_attach_iodimm2(fio_iodimm_selector_t* device, fio_iodimm_attach_req2_t* request, fio_progress_callback2_t* callback);

/**
 * \brief Detach the iodrive to allow operations such as low-level format. This function is thread-safe and may be
 *   called on multiple ioDimm's in parallel.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request the detach request structure.  See the header (iodimm_types.h) for more information on how to populate this struct. *
 * \param callback structure (optional, may be NULL).  See the header (common_types.h) for more information on how to populate this struct.
 * \return See the fio_iodimm_detach_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_detach_status_t
fio_detach_iodimm2(fio_iodimm_selector_t* device, fio_iodimm_detach_req2_t* request, fio_progress_callback2_t* callback);

/**
 * \brief Update the ioDrive firmware using the given firmware file. This function is thread-safe and may be called
 *  on multiple ioDimm's in parallel.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request the update request structure.  See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param callback structure (optional, may be NULL).  See the header (common_types.h) for more information on how to populate this struct.
 * \return See the fio_iodimm_update_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_update_status_t
fio_update_iodimm_firmware2(fio_iodimm_selector_t* device, fio_iodimm_update_req2_t* request, fio_progress_callback2_t* callback);

/**
 * \brief Get the firmware version for the selected device in the given FFF file. This can be used to tell whether a
 *   device is up to date with respect to a given fff file.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request the get firmware request structure.  See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param firmware_ver OUTPUT: an allocated structure to place the firmware version in.
 * \return See the fio_iodimm_fff_ver_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_fff_ver_status_t
fio_get_fff_firmware_ver2(fio_iodimm_selector_t* device, fio_iodimm_fff_ver_req2_t* request, fio_firmware_version_t* firmware_ver);

/**
 * \brief Get the firmware barrier version--if any--given the current firmware version.
 *   You must upgrade first to the barrier version before upgrading to the final version.
 *   This is useful to determine if one can upgrade directly from the current version to a final version.
 * \param barrier IN/OUT: an allocated structure to specify the current version and to receive the barrier version.
 * \return zero if a barrier version was found for the current version. Nonzero otherwise.
 */
DllExport
uint8_t fio_iodimm_update_get_firmware_barrier_version(fio_firmware_barrier_t* barrier);

/**
 * \brief Perform a low-level format on the device. The device must be in a detached state.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request the format request structure.  See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param callback structure (optional, may be NULL).  See the header (common_types.h) for more information on how to populate this struct.
 * \return See the fio_iodimm_format_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_format_status_t
fio_format_iodimm2(fio_iodimm_selector_t* device, fio_iodimm_format_req2_t* request, fio_progress_callback2_t* callback);

/**
 * \brief Enable or disable preallocation of memory for the given ioDimm.  Enabling preallocation will cause the driver
 *      to allocate worst case memory usage for this ioDimm, which will allow this ioDimm to be used as swap space.
 * \param device Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \param request prealloc settings. See the header (iodimm_types.h) for more information on how to populate this struct.
 * \return See the fio_iodimm_prealloc_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_prealloc_status_t
fio_set_iodimm_preallocation(fio_iodimm_selector_t* device, fio_iodimm_prealloc_req_t* request);

/**
 * \brief Change the enabled state of the lifetime extender.
 *
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.
 *       If the device was not previously opened via open_iodimm_handle, this function internally opens and closes it.
 * \param enable pointer to a structure with the desired lifetime extender enable parameters set.
 *
 * \return See the fio_extender_enable_status_t declaration for return value descriptions.
 */
DllExport
fio_iodimm_extender_status_t
fio_iodimm_enable_lifetime_extender(fio_iodimm_selector_t *input, fio_iodimm_lifetime_extender_enable_t *enable);



/************************************************************************************
 *                               DEPRECATED FUNCTIONS                               *
 ************************************************************************************/
#if FIO_SDK_DEPRECATED == 1 /* defined in version.h */

/**
 * \brief Attach the ioDimm to allow it to function as a block device. This function is thread-safe and may be called 
 *      on multiple ioDimm's in parallel.
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct. 
 * \return see fio_iodimm_attach_rsp_t documentation for details.
*/
DllExport
fio_iodimm_attach_rsp_t
fio_attach_iodimm(fio_iodimm_attach_req_t* input);

/**
 * \brief Detach the iodrive to allow operations such as low-level format. This function is thread-safe and may be 
 *   called on multiple ioDimm's in parallel.
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct. 
 * \return See the fio_iodimm_detach_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_detach_rsp_t
fio_detach_iodimm(fio_iodimm_detach_req_t* input);

/**
 * \brief Update the ioDrive firmware using the given firmware file. This function is thread-safe and may be called 
 *  on multiple ioDimm's in parallel.
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.    
 * \return See the fio_iodimm_update_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_update_rsp_t
fio_update_iodimm_firmware(const fio_iodimm_update_req_t* input);

/**
 * \brief Get the firmware version for the selected device in the given FFF file. This can be used to tell whether a 
 *   device is up to date with respect to a given fff file.
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.  
 * \return See the fio_iodimm_fff_ver_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_fff_ver_rsp_t
fio_get_fff_firmware_ver(fio_iodimm_fff_ver_req_t* input);

/**
 * \brief Perform a low-level format on the device. The device must be in a detached state.
 * \param input Device selector. See the header (iodimm_types.h) for more information on how to populate this struct.  
 * \return See the fio_iodimm_format_rsp_t declaration for return value descriptions.
*/
DllExport
fio_iodimm_format_rsp_t
fio_format_iodimm(fio_iodimm_format_req_t* input);

#endif /* FIO_SDK_DEPRECATED */

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_IODIMM_OPERATIONS_H_ */
