/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_IODIMM_TYPES_H_
#define _INCLUDE_FIO_PUBLIC_IODIMM_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/common_types.h>
#include <fio/public/dyn_struct_alloc.h>

/** \brief Size of the driver version string */
#define FIO_DRIVER_VER_SIZE          255
/** \brief Size of the device label string */
#define FIO_IODIMM_LABEL_SIZE        255
/** \brief Size of the device name string */
#define FIO_IODIMM_DEVICE_NAME_SIZE  100
/** \brief Size of the block device name string */
#define FIO_IODIMM_BLK_DEV_NAME_SIZE 100
/** \brief Size of the alt part number string */
#define FIO_IODIMM_ALT_PART_NUM_SIZE 255
/** \brief Serial number string length */
#define FIO_IODIMM_SERIAL_NUM_STR_LEN 129

/** \brief Size of the fio-format generated unique id (uuid) string */
#define FIO_IODIMM_UUID_SIZE 37

/******************************************
 *          Shared ioDimm Types            *
 ******************************************/

/**
 * \brief Device selector mode enumeration.  This is used when opening a device handle to indicate how
 *  the handle will be opened.
 */
typedef enum
{
    FIO_IODIMM_SELECT_NAME = 0,      /**< Select a device by its name.  ex: /dev/fct0.  This mode is only available when the driver is loaded. */
    FIO_IODIMM_SELECT_INDEX = 1,     /**< Select a device by index.  ex: 0 = /dev/fct0. This mode is only available when the driver is loaded. */
    FIO_IODIMM_SELECT_HANDLE = 2,    /**< DEPRECATED.  Prior to SDK v 3: Not used explicitly.  If you open a device via open_iodimm_handle,
                                       the device selector is converted to this selection mode, indicating to the
                                       SDK that the handle is already opened.  This mode is also incremented to
                                       keep track of how many times it has been opened.  Attempting to compare against
                                       this value will lead to inconsistent results.*/
    FIO_IODIMM_SELECT_PCI = 3,       /**< (Linux Only) Select a device by its PCI address (domain, bus, device and function).
                                       Only a small number of functions support this selector mode and functions supporting this mode will be
                                       documented as supporting PCI based selectors.  This mode is only available when the driver is NOT loaded. */
    FIO_IODIMM_SELECT_PCI_STRING = 4 /**< (Linux Only) Select a device by the string version of its PCI address. ex: 6:0:0.0.
                                       Only a small number of functions support this selector mode and functions supporting this mode will be
                                       documented as supporting PCI based selectors.  This mode is only available when the driver is NOT loaded. */
} fio_iodimm_selector_mode;

/**
 * \brief Select an ioDimm. This structure is used as the input to most operations. Depending on the selector mode,
 *  one of the members must be populated with the correct value.
*/
typedef struct fio_iodimm_selector
{
    uint32_t                 lib_version;      /**< REQUIRED: Fusion SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_mode mode;             /**< REQUIRED: Specifies which of the remaining members will be populated */
    uint32_t                 index;            /**< CONDITIONAL: Device Index (ex: 0), populate if the selector mode is FIO_IODIMM_SELECT_INDEX */
    const char*              device_name;      /**< CONDITIONAL: Device Name (ex: /dev/fct0) , populate if the selector mode is FIO_IODIMM_SELECT_NAME */
    void*                    device_handle;    /**< INTERNAL: Internally used if device is opened via a non-PCI mode in open_iodimm_handle */
    uint8_t                  open_count;       /**< INTERNAL: Internally used to track open and close calls on this selector. (available SDK v 3+) */
    const char*              pci_address;      /**< CONDITIONAL: PCI Address (ex: 6:0:0.0), populate if the selected mode is FIO_IODIMM_SELECT_PCI_ADDRESS. (available SDK v 3+) */
    uint16_t                 pci_domain;       /**< CONDITIONAL: PCI Domain Address (ex: 6), populate if the selected mode is FIO_IODIMM_SELECT_PCI. (available SDK v 3+) */
    uint8_t                  pci_bus;          /**< CONDITIONAL: PCI Bus Address (ex: 0), populate if the selected mode is FIO_IODIMM_SELECT_PCI. (available SDK v 3+) */
    uint8_t                  pci_device;       /**< CONDITIONAL: PCI Device Address (ex: 0), populate if the selected mode is FIO_IODIMM_SELECT_PCI. (available SDK v 3+) */
    uint8_t                  pci_function;     /**< CONDITIONAL: PCI Function Address (ex: 0), populate if the selected mode is FIO_IODIMM_SELECT_PCI. (available SDK v 3+) */
    void*                    pci_info;         /**< INTERNAL: Internally used if device is opened via a PCI mode in open_iodimm_handle. */
    uint8_t                  csr_count;        /**< INTERNAL: Internally used to track open and close CSR calls on this selector. */
    void*                    csr_handle;       /**< INTERNAL: Internally used to hold CSR info if the handle is used to open CSR. */
} fio_iodimm_selector_t;

/**
 * \brief Used to iterate over ioDimms.  The structure should be initialized by FIO_STRUCT_INIT.  All fields of the structure are
 * used internally by first_iodimm_handle() and next_iodimm_handle().  Fields should not be set by the caller.
 */
typedef struct fio_iodimm_iterator
{
    uint32_t                    lib_version;    /**< Fusion SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_mode    mode;           /**< INTERNAL: The mode being used to iterate over the devices. (FIO_IODIMM_SELECT_INDEX or FIO_IODIMM_SELECT_PCI)  */
    void*                       position;       /**< INTERNAL: The current position of the iterator. */
    fio_status_result_t         result;         /**< OUTPUT: The current state of the iterator. FIO_STATUS_SUCCESS indicates the last call to first_iodimm_handle() or next_iodimm_handle()
                                                          returned a valid ioDimm.  FIO_STATUS_DEVICE_NOT_FOUND indicates the end of the list has been reached.  All other
                                                          values indicate an error. */
} fio_iodimm_iterator_t;

/******************************************
 *               Status Types              *
 ******************************************/
/** \brief Enumeration of potential adapter types that may contain an ioDimm */
typedef enum
{
    FIO_IODIMM_ADAPTER_TYPE_IODRIVE = 0,            /**< Adapter for dimm is a standard ioDrive */
    FIO_IODIMM_ADAPTER_TYPE_IODRIVE_LOWPRO2 = 1,    /**< Adapter for dimm is a LowPro2 ioDrive or ioMono */
    FIO_IODIMM_ADAPTER_TYPE_IODRIVE_DUO = 2,        /**< Adapter for dimm is an ioDrive Duo */
    FIO_IODIMM_ADAPTER_TYPE_IOSAN = 3,              /**< Adapter for dimm is an ioSAN */
    FIO_IODIMM_ADAPTER_TYPE_UNKNOWN = 4,            /**< Could not determine the adapter type */
    FIO_IODIMM_ADAPTER_TYPE_IOOCTAL = 5,            /**< Adapter for dimm is an ioOctal */
} fio_iodimm_adapter_type_t;

/** \brief Information about an ioDimm's adapter */
typedef struct fio_iodimm_adapter_info
{
    uint32_t                    lib_version;            /**< Fusion SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_adapter_type_t   type;                   /**< Adapter type for this ioDimm */
    uint32_t                    port_number;            /**< Indicates the ioDimm's port within the adapter.  If it is
                                                             the only ioDimm in the adapter, it will be set to 0. */
    uint64_t                    serial_number;          /**< Adapter serial number */
    uint8_t                     external_power_present; /**< bool: Status of external power connection */

    /* SDK API Version 2 and above: */
    fio_pciexpress_info_t       pcie;                   /**< PCI Express information about this adapter */
    fio_iodimm_compatibility_t  bandwidth_compatibility;/**< PCIe link bw compatibility of this adapter in the motherboard slot */
    fio_iodimm_compatibility_t  power_compatibility;    /**< Wattage compatibility of this adapter in the motherboard slot */
    uint8_t                     power_monitor_enabled;  /**< bool: 0 if power monitor is either disabled or not present. */
    float                       power_limit_watts;      /**< The current setting for power-throttling limit. */

    uint8_t                     powerloss_protect_avail;    /**< bool: 0 if card and firmware DO NOT support power loss protection (powercut) */
    uint8_t                     powerloss_protect_enabled;  /**< bool: 0 if power loss protection is NOT enabled. */
    /* SDK API Version 4 and above */
    uint8_t                     required_is_valid;      /**< If 1 then 'required_power' and 'required_bandwidth' are valid. */
    float                       required_power_watts;   /**< Power required by this adapter to avoid power throttling. */
    float                       required_bandwidth_MBps;/**< Bandwidth required to avoid performance limiting. */
    /* SDK API Version 7 and above */
    char                        serial_number_str[FIO_IODIMM_SERIAL_NUM_STR_LEN];
} fio_iodimm_adapter_info_t;

/** \brief Various product information for an ioDimm */
typedef struct fio_iodimm_product_info
{
    uint32_t   lib_version;             /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint64_t   serial_number;           /**< Displayable serial number */
    uint32_t   date;                    /**< Board manufacturing date */
    char       manufacturer_name[255];  /**< Manufacturer name (8-bit ASCII) */
    char       part_number_int[255];    /**< Internal part number (8-bit ASCII) */
    char       part_number_ext[255];    /**< External part number (8-bit ASCII). If empty string, fio-update-mid is proscribed. */
    char       name[255];               /**< Product name (8-bit ASCII). If empty string, fio-update-mid is proscribed. */
    char       subsys_name[255];        /**< Subsystem product name (8-bit ASCII) */
    char       fpga_id[255];            /**< FPGA ID */
    uint32_t   factory_fw_version;      /**< Currently installed firmware version */
    /* SDK version 3: */
    uint8_t    product_info_invalid;    /**< If 1, then part_number_ext and name invalid: try fio-update-mid on this card. */
    /* SDK version 4: */
    uint32_t   rated_endurance_tbytes;  /**< Rated write endurance of this card, in Terabytes, or 0 if not found. */
    uint32_t   rated_endurance_days;    /**< Rated write endurance of this card, in days or 0 if not found. */
    /* SDK version 6: */
    int64_t    first_use_date;          /**< Date the card was first used by the customer (posix epoch format). Zero = unavailable. */
    /* SDK version 7: */
    char       serial_number_str[FIO_IODIMM_SERIAL_NUM_STR_LEN];
} fio_iodimm_product_info_t;

/** \brief Enumeration of states that the ioDimm may be in */
typedef enum
{
    FIO_IODIMM_STATE_DETACHED = 0,          /**< Device is currently detached */
    FIO_IODIMM_STATE_ATTACHING = 1,         /**< Device is performing an attach operation */
    FIO_IODIMM_STATE_SCANNING_METADATA = 2, /**< Device is scanning its metadata during an attach operation */
    FIO_IODIMM_STATE_SCANNING_DATA = 3,     /**< Device is scanning its data during an attach operation */
    FIO_IODIMM_STATE_ATTACHED = 4,          /**< Device is currently attached and operational */
    FIO_IODIMM_STATE_DETACHING = 5,         /**< Device is performing a detach operation */
    FIO_IODIMM_STATE_STOPPING = 6,          /**< Device is in the process of shutting down */
    FIO_IODIMM_STATE_STOPPED = 7,           /**< Device has been shut down */
    FIO_IODIMM_STATE_MINIMAL = 8,           /**< Device is running in a minimal state. See FIO_MINMODE_REASON enum */
    FIO_IODIMM_STATE_FAILED = 9,            /**< Device has had an internal failure and is not operational */
    FIO_IODIMM_STATE_UNKNOWN = 10           /**< Device state could not be determined */
} fio_iodimm_state_t;

/** \brief Enumeration of flashback statuses */
typedef enum
{
    FIO_IODIMM_FLASHBACK_STATUS_UNAVAILABLE = 0,    /**< Flashback status could not be determined. */
    FIO_IODIMM_FLASHBACK_STATUS_NOT_ACTIVE = 1,     /**< Flashback is not active on any bank. */
    FIO_IODIMM_FLASHBACK_STATUS_ACTIVE = 2,         /**< Flashback is active on at least one bank. */
} fio_iodimm_flashback_status_t;

/** \brief State of flashback on an ioDimm */
typedef struct
{
    uint32_t                        lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_flashback_status_t   status;         /**< The status of flashback on the device */
    uint8_t                         failed;         /**< Whether flashback has failed (could not substitute
                                                       again on at least one bank). */
} fio_iodimm_flashback_t;

/** \brief Reasons that an ioDimm may have gone into minimal mode. */
typedef enum
{
    FIO_MINMODE_REASON_UNKNOWN = 0,             /**< Reason unknown, or the device is not currently in minimal mode */
    FIO_MINMODE_REASON_FW_OUT_OF_DATE = 1,      /**< Firmware is out of date */
    FIO_MINMODE_REASON_INSUFFICIENT_POWER = 2,  /**< Adapter power needs to be plugged in */
    FIO_MINMODE_REASON_DUAL_PLANE_FAIL = 3,     /**< Could not enter dual-plane mode, although user-specified */
    FIO_MINMODE_REASON_FORCED = 4,              /**< Forced into minimal mode by the user (module parameter) */
    FIO_MINMODE_REASON_INTERNAL = 5,            /**< Internal error forced the device into minimal mode */
    FIO_MINMODE_REASON_CARD_LIMIT_EXCEEDED = 6, /**< Product-specific device limit has been exceeded in this machine */
    FIO_MINMODE_REASON_NOT_IN_MINMODE = 7,      /**< Not currently in minimal mode. */
    FIO_MINMODE_REASON_UNSUPPORTED_OS = 8,      /**< The device will not function on this operating system. */
    FIO_MINMODE_REASON_INSUFFICIENT_MEMORY = 9,  /**< There is not enough memory to load the driver */
    FIO_MINMODE_REASON_SMP_BOOTLOADER_MODE = 10  /**< SMP is in bootloader mode */
} fio_iodimm_minmode_reason_t;

/** \brief Current state of an ioDimm. */
typedef struct
{
    uint32_t                    lib_version;          /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_state_t          state;                /**< State of the driver for this device */
    fio_iodimm_minmode_reason_t minimal_mode_reason;  /**< If the state is minimal mode, contains the reason */
} fio_iodimm_state_info_t;

/** \brief Levels of write throttling that an ioDimm may enter */
typedef enum fio_iodimm_write_throttling_level
{
    FIO_WRITE_THROTTLE_NONE = 0,        /**< Device is not currently throttling writes */
    FIO_WRITE_THROTTLE_PARTIAL = 1,     /**< Device is throttling writes to a very low speed */
    FIO_WRITE_THROTTLE_COMPLETE = 2     /**< Device has stopped all writes */
} fio_iodimm_write_throttling_level_t;

/** \brief Reasons why an ioDimm may begin write throttling */
typedef enum fio_iodimm_write_throttling_reason
{
    FIO_WRITE_THROTTLE_REASON_NONE = 0,
    FIO_WRITE_THROTTLE_REASON_USER = 1,             /**< User call caused the current write-throttling mode. */
    FIO_WRITE_THROTTLE_REASON_NO_MD_BLOCKS = 2,     /**< Out of metadata blocks */
    FIO_WRITE_THROTTLE_REASON_NOMEM = 3,            /**< Out of memory (RAM) */
    FIO_WRITE_THROTTLE_REASON_DEAD_DIE = 4,         /**< Dead chip die or permanent parity substitution */
    FIO_WRITE_THROTTLE_REASON_WEAROUT = 5,          /**< Too many failed LEBs to continue unthrottled */
    FIO_WRITE_THROTTLE_REASON_ADAPTER_POWER = 6,    /**< Adapter power isn't plugged in. Note: This is not the
                                                       behavior for all devices; some devices start up in minimal mode n this case. */
    FIO_WRITE_THROTTLE_REASON_INTERNAL = 7,         /**< Internal failure caused throttled mode */
    FIO_WRITE_THROTTLE_REASON_LIMIT_POWER = 8,      /**< Attempt to minimize power consumption */
    FIO_WRITE_THROTTLE_REASON_GROOM_FAILS = 9,      /**< Groomer could not free enough blocks to continue. */
} fio_iodimm_write_throttling_reason_t;

/** \brief Current write-throttling state of an ioDimm */
typedef struct fio_iodimm_write_throttling
{
    uint32_t                                lib_version;  /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_write_throttling_level_t     state;        /**< Level of current write throttling */
    fio_iodimm_write_throttling_reason_t    reason;       /**< Reason for write throttling */
} fio_iodimm_write_throttling_t;

/** \brief Peak values that can be cleared. These can be OR'd together to clear multiples at once. */
typedef enum
{
    FIO_PEAKS_NONE          = 0,        /**< No peaks */
    FIO_PEAKS_MAX_VOLTS     = 1 << 0,   /**< Maximum volts peak */
    FIO_PEAKS_MIN_VOLTS     = 1 << 1,   /**< Minimum volts peak */
    FIO_PEAKS_MAX_WATTS     = 1 << 2,   /**< Maximum watts peak */
    FIO_PEAKS_ALL           = 0xff      /**< All peaks */
} fio_iodimm_power_usage_peaks_t;

/** \brief Struct representing power monitor information from the driver */
typedef struct fio_iodimm_power_usage
{
    uint32_t    lib_version;        /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint8_t     is_valid;           /**< Whether or not the power usage could be determined.  a value of 0 indicates
                                       that it could not be determined and the other values in this struct are bad. */
    float       bus_volts;          /**< The voltage on the PCIE 12V bus. */
    float       bus_peak_volts;     /**< The max voltage seen on the PCIE 12V bus since last cleared. */
    float       bus_min_volts;      /**< The min voltage seen on the PCIE 12V bus since last cleared. */
    float       bus_watts;          /**< The wattage being drawn on the PCIE 12V bus. */
    float       bus_peak_watts;     /**< The peak wattage seen on the PCIE 12V bus since last cleared. */
    float       bus_amps;           /**< The current value of amperage flowing in the PCIE 12V bus. */
    float       bus_peak_amps;         /**< The current value of amperage flowing in the PCIE 12V bus. */
} fio_iodimm_power_usage_t;

/** \brief Setpoints for the power monitoring system */
typedef struct fio_iodimm_power_setpoints
{
    uint32_t    lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    float       low_watts;      /**< Low setpoint for watts */
    float       high_watts;     /**< High setpoint for watts */
} fio_iodimm_power_setpoints_t;

/** \brief Hardware alert bitfield definition. */
typedef enum
{
    FIO_HW_ALERT_NONE           = 0x00,
    FIO_HW_ALERT_TEMPERATURE    = 0x01,   /**< Temperature threshold exceeded. */
    FIO_HW_ALERT_VCCINT         = 0x02,   /**< Internal voltage threshold exceeded. */
    FIO_HW_ALERT_VCCAUX         = 0x04,   /**< Auxiliary voltage threshold exceeded. */
    FIO_HW_ALERT_OVERTEMP       = 0x08,   /**< Extreme temperature threshold exceeded. */
} fio_iodimm_hw_alert_type_t;

/** \brief Hardware alerts from sysmon. */
typedef struct fio_iodimm_hw_alerts
{
    uint32_t    lib_version;   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_hw_alert_type_t type;  /**< The hw_alert_type for the priority alert. */
    fio_iodimm_hw_alert_type_t active_mask;  /**< A bit field that shows all alerts that are currently active. */
    int64_t     timestamp;     /**< The time the alert occurred. */
    float       current_value; /**< The current value for the priority alert type. */
    float       setpoint_low;  /**< The low setpoint for the priority alert type. */
    float       setpoint_high; /**< The high setpoint for the priority alert type. */
} fio_iodimm_hw_alerts_t;

/** \brief Capacity history data point. */
typedef struct fio_iodimm_capacity_entry
{
    uint64_t capacity_bytes;    /**< Total capacity of the device in bytes at 'timestamp' point in time, including reserves. */
    int64_t  timestamp;         /**< The time the measurement was taken. */
    /* SDK fio_iodimm_history_table_t Version 3 and higher. */
    float    health_percent;    /**< The ioDrive health percentage at 'timestamp' point in time. */
} fio_iodimm_capacity_entry_t;

/** \brief Write history data point. */
typedef struct fio_iodimm_usage_entry
{
    uint64_t usage_bytes;   /**< Number of physical bytes written to the device since its origin, at 'timestamp' time. */
    int64_t  timestamp;     /**< The time the measurement was taken. */
} fio_iodimm_usage_entry_t;

/** \brief Temperature history data point. */
typedef struct fio_iodimm_temperature_entry
{
    time_t start_timestamp; /**< Start of the interval in which the measurement was taken. */
    time_t end_timestamp;   /**< End of the interval in which the measurement was taken. */
    int8_t max_degrees_C;   /**< Maximum temperature observed over the interval, in degrees C */
    int8_t avg_degrees_C;   /**< Average temperature observed over the interval, in degrees C */
} fio_iodimm_temperature_entry_t;

/** \brief Types of history entries stored in the ioDimm's non-volatile data segment */
typedef enum
{
    FIO_IODIMM_HISTORY_TYPE_UNKNOWN = 0,
    FIO_IODIMM_HISTORY_TYPE_CAPACITY = 1,          /**< Entries represent fio_iodimm_capacity_entry_t structs. */
    FIO_IODIMM_HISTORY_TYPE_PHYSICAL_READS = 2,    /**< Entries represent fio_iodimm_usage_entry_t structs. */
    FIO_IODIMM_HISTORY_TYPE_PHYSICAL_WRITES = 3,   /**< Entries represent fio_iodimm_usage_entry_t structs. */
    FIO_IODIMM_HISTORY_TYPE_FPGA_TEMPERATURE = 5,  /**< Entries represent fio_iodimm_temperature_entry_t structs. */
    FIO_IODIMM_HISTORY_TYPE_BOARD_TEMPERATURE = 6, /**< Entries represent fio_iodimm_temperature_entry_t structs. */
    FIO_IODIMM_HISTORY_TYPE_MAX
} fio_iodimm_history_type_t;

/** \brief Structure used to fetch historical data points about various aspects of the device. */
typedef struct
{
    uint32_t  lib_version;        /**< IN: Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_history_type_t  entrytype; /**< IN: Type of entry to retrieve. */
    uint32_t  size_entries;       /**< IN: Number of entry locations the user has allocated in the table. */
    int64_t   begin_timestamp;    /**< IN: Timestamp of the first desired entry. Set to 0 to start at first history entry. */
    int64_t   end_timestamp;      /**< IN: Timestamp of last desired entry. Set to 0 to end at the last history entry */
    uint32_t  num_entries;        /**< OUT: Number of entries actually filled in this table. */
    int64_t   first_timestamp;    /**< OUT: Timestamp of the first entry in the table. */
    int64_t   last_timestamp;     /**< OUT: Timestamp of the last entry in the table. */
    uint32_t  remaining;          /**< Must be set to 0 on the first call. This will be non-zero if not all entries
                                        were retrieved. In that case, another call must be made. */
    union
    {
        fio_iodimm_capacity_entry_t*    capentry;
        fio_iodimm_usage_entry_t*       usgentry;
        fio_iodimm_temperature_entry_t* tmpentry;
    } uu;                         /**< OUT: Union of the desired entry types, pre-allocated to hold entries */
    /* SDK Version 3: */
    int64_t origin_timestamp;     /**< OUT: Timestamp that the history first began for this ioDrive. */
} fio_iodimm_history_table_t;

/** \brief Usage data for an ioDimm */
typedef struct fio_iodimm_usage
{
    uint32_t lib_version;                   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint64_t total_logical_bytes_written;   /**< Deprecated. Always returns 0. */
    uint64_t total_logical_bytes_read;      /**< Deprecated. Always returns 0. */
    uint64_t session_write_ops;             /**< Number of write operations since last attach */
    uint64_t session_read_ops;              /**< Number of read operations since last attach */
    /* SDK Version 3: */
    uint64_t total_physical_bytes_written;  /**< Bytes actually written to the NAND array, including driver overhead. */
    uint64_t total_physical_bytes_read;     /**< Bytes actually read from the NAND array, including driver overhead. */
    /* SDK Version 4: */
    uint64_t current_ram_usage_bytes;    /**< The current amount of memory in bytes allocated for this device. */
    uint64_t high_ram_usage_bytes;       /**< The highest amount of memory in bytes allocated for this device since driver load. */
    /* SDK Version 6: */
    int32_t attach_seconds;                 /**< Number of seconds the device has been attached. */
} fio_iodimm_usage_t;

/** \brief Structure containing various performance data for an ioDimm */
typedef struct fio_iodimm_temperature_setpoints
{
    uint32_t    lib_version;    /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    int8_t      high_c;         /**< Upper setpoint in degrees celsius */
    int8_t      low_c;          /**< Lower setpoint in degrees celsius */
    /* SDK API Version 2 and above: */
    int8_t      warn_c;         /**< Warning threshold for this temperature. Above this, warn user. */
} fio_iodimm_temperature_setpoints_t;

/** \brief Settings for an LED */
typedef enum fio_iodimm_led_status
{
    FIO_IODIMM_LED_STATUS_OFF = 0,  /**< LED is off */
    FIO_IODIMM_LED_STATUS_ON = 1,   /**< LED is on */
    FIO_IODIMM_LED_STATUS_BLINK = 2 /**< LED is blinking at a fixed interval */
} fio_iodimm_led_status_t;

/** \brief Status of the LED's on the ioDimm */
typedef struct fio_led_status
{
    uint32_t                lib_version;            /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_led_status_t red;                    /**< Status of the red LED */
    fio_iodimm_led_status_t yellow;                 /**< Status of the yellow LED */
    fio_iodimm_led_status_t green;                  /**< Status of the green LED */
    fio_iodimm_led_status_t external;

    uint8_t                 yellow_duty_percent;
    uint16_t                yellow_period_msec;

    uint8_t                 green_duty_percent;
    uint16_t                green_period_msec;

    uint8_t                 extern_duty_percent;
    uint16_t                extern_period_msec;
} fio_led_status_t;

/** \brief Health states for an ioDimm */
typedef enum
{
    FIO_IODIMM_HEALTH_UNKNOWN = 0,          /**< Health of the device is unknown */
    FIO_IODIMM_HEALTH_HEALTHY = 1,          /**< Device is healthy */
    FIO_IODIMM_HEALTH_WARNING = 2,          /**< Device is getting close to entering reduced write mode. */
    FIO_IODIMM_HEALTH_REDUCED_WRITE = 3,    /**< Device has entered reduced write mode to preserve the flash from
                                               further wearout. */
    FIO_IODIMM_HEALTH_READ_ONLY = 4,        /**< Device has entered read-only mode to preserve the flash from
                                               further wearout.*/
} fio_iodimm_health_status_t;

/** \brief Health of an ioDimm */
typedef struct fio_iodimm_health
{
    uint32_t                    lib_version;              /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    float                       capacity_health_percent;  /**< Percent of reserve space left on the ioDimm */
    fio_iodimm_health_status_t  status;                   /**< Health state of the ioDimm as contained in the enum  */
} fio_iodimm_health_t;

/** \brief Device label */
typedef struct fio_iodimm_device_identifiers
{
    uint32_t    lib_version;                                /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    char        dev_name[FIO_IODIMM_DEVICE_NAME_SIZE];      /**< Character buffer where device name will be stored */
    char        blkdev_name[FIO_IODIMM_BLK_DEV_NAME_SIZE];  /**< Character buffer where block device name will be
                                                               stored (N/A on Windows) */
    char        label[FIO_IODIMM_LABEL_SIZE];               /**< Character buffer where label will be stored */
} fio_iodimm_device_identifiers_t;

/** \brief Static format info of the ioDimm */
typedef struct fio_iodimm_static_format_info
{
    uint32_t lib_version;               /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint64_t max_format_size_mb;        /**< MiBytes! "Default" format size of the device in MiB, not "max" as implied.
                                             (ie: default sector size, no under/over formatting). */
    /* SDK API Version 2 and above: */
    uint32_t default_sector_size_bytes; /**< The default sector size to which the device will be formatted. */
    uint32_t min_sector_size_bytes;     /**< Minimum supported sector size in bytes. */
    uint32_t max_sector_size_bytes;     /**< Maximum supported sector size in bytes. */
    uint8_t  power_of_two_sector_size;  /**< Set to 1 if the sector size is required to be a power of 2. */
    uint32_t sector_size_multiplier;    /**< If non-zero, the sector size must be a multiple of this number. */

    /* SDK API Version 3 and above: */
    uint8_t  disallow_overformat;       /**< If non-zero then do not allow formatting larger than 'max_format_size_mb'.*/

    /* SDK API Version 4 and above: */
    uint8_t  nv_data_scan_skipped;      /**< OUT: Value is non-zero when the NV data scan was skipped when the driver loaded. */
    uint8_t  odometer_reset;            /**< OUT: Value is non-zero when the drive has been formatted after the NV data scan
                                                  was skipped. (Warranty is now void).*/
    /* SDK API Version 6 and above */
    uint8_t  assume_pstore;             /**< [in] bool: 1=return Default size assuming creation of a default-sized Perisistent Parameter Store. */
} fio_iodimm_static_format_info_t;

typedef enum fio_format_type
{
    FIO_FORMAT_TYPE_STANDARD = 0, /**< Format should produce a standard block device. */
    FIO_FORMAT_TYPE_CACHE    = 1, /**< Format for use as a directCache cache device. */
    FIO_FORMAT_TYPE_UNKNOWN  = 2, /**< Target of this format is unknown. */
} fio_format_type_t;

/** \brief Formatted device sizes calculations for an ioDimm */
typedef struct fio_iodimm_format_sizes
{
    uint32_t lib_version;                   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint32_t sector_size_bytes;             /**< [in] Sector size from which to calculate the sizes below. 0=default sector size. */
    uint64_t max_format_size_mibytes;       /**< Max formattable size with the given sector_size_bytes. Specifying a larger size
                                                 than this implies sparse-formatting. */
    uint64_t default_format_size_mibytes;   /**< Advertized size of ioDimm: size with no over/under formatting.*/

    /* SDK API Version 3 and above: */
    uint8_t  disallow_overformat;           /**< If non-zero then do not allow formatting larger than 'max_format_size_mb'.*/
    /* SDK API Version 6 and above */
    uint64_t max_sparse_size_mibytes;       /**< Max size that can be available if sparse formatting. */
    uint32_t dflt_pstore_entries;           /**< The default Perisistent Parameter Store size, in number of key/value entries it can hold.
                                                If a PPS is created in the format request then the actual resultant format size will be
                                                dflt_param_store_entries * "bytes per pps entry" smaller than these returned values for
                                                max_format_size_mibytes and max_sparse_size_mibytes. */
    uint32_t dflt_pstore_value_max_bytes;   /**< The default PPS maxinum size for an entry's value. */
    fio_format_type_t format_target;        /**< [in] The target use of this ioDrive: ie: standard block device, cache device, etc. */
} fio_iodimm_format_sizes_t;

/** \brief Current format information describing the current formatting of the ioDimm */
typedef struct fio_iodimm_dynamic_format_info
{
    uint32_t lib_version;           /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint64_t cur_format_size_mb;    /**< MiBytes! Current formatted size of the device in MiB */
    /* SDK API Version 2 and above: */
    uint16_t format_version;        /**< Or 0 if the drive is not formatted. */
    uint64_t blockdev_num_sectors;  /**< The number of sectors presented to the OS block device layer. */
    uint32_t sector_size_bytes;     /**< The sector (packet) size at which this drive is formatted. */
    char     format_uuid[FIO_IODIMM_UUID_SIZE]; /**< The uuid given to the volume during low-level fio-format */
    /* SDK API Version 6 and above */
    fio_format_type_t format_target;/**< The target use of this ioDrive: ie: standard block device, cache device, etc. */
} fio_iodimm_dynamic_format_info_t;

/** \brief Driver's version info */
typedef struct fio_iodimm_driver_ver_info
{
    uint32_t lib_version;                   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    char     version[FIO_DRIVER_VER_SIZE];  /**< Buffer where driver version is stored */
    /* SDK API Version 2 and above: */
    uint32_t major;                         /**< Driver major version */
    uint32_t minor;                         /**< Driver minor version */
    uint32_t micro;                         /**< Driver micro version */
    uint32_t build;                         /**< Driver build number */
} fio_iodimm_driver_ver_info_t;

/** \brief Alternate driver information */
typedef struct fio_iodimm_alt_info
{
    uint32_t lib_version;                                 /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    char     alt_part_num[FIO_IODIMM_ALT_PART_NUM_SIZE];  /**< Buffer where alternate part number is stored */
} fio_iodimm_alt_info_t;

/** \brief Firmware barrier version information */
typedef struct fio_firmware_barrier
{
    uint32_t lib_version;                   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_firmware_version_t current;         /**< IN: Current version of the firmware */
    fio_firmware_version_t barrier;         /**< OUT: Barrier version to the current version, if any. */
    fio_iodimm_driver_ver_info_t driver;    /**< OUT: Driver version corresponding closest to firmware barrier version*/
} fio_firmware_barrier_t;

/** \brief General ioDimm configuration settings */
typedef struct fio_iodimm_settings
{
    uint32_t lib_version;                   /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t  is_preallocation_enabled;      /**< Whether or not full memory preallocation is enabled on this iodimm. */
} fio_iodimm_settings_t;

/** \brief Current temperatures */
typedef struct fio_iodimm_temperatures
{
    uint32_t lib_version;       /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t  ambient_temp_c;    /**< Board ambient temperature in degrees celsius */
    uint8_t  engine_temp_c;     /**< Engine temperature in degrees celsius */
} fio_iodimm_temperatures_t;

/******************************************
 *             Attach Types               *
 ******************************************/

/** \brief Bitmask of flags that influence the attach process. */
typedef enum fio_iodimm_attach_flags
{
    FIO_ATTACH_FLAG_DEFAULT         = 0x00,         /**< Attempt attach, with full-scan if required. */
    FIO_ATTACH_FLAG_CLEAN_ONLY      = 0x01,         /**< Attach only if clean shutdown and full scan is not required. */
    FIO_ATTACH_FLAG_FORCE_RESCAN    = 0x02,         /**< Always force a full scan during attach. */
} fio_attach_flags_t;

/** \brief Structure used to request an ioDimm to attach */
typedef struct fio_iodimm_attach_req2
{
    uint32_t            lib_version;                /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    /* SDK Version 6 and above. */
    fio_attach_flags_t  flags;                      /**< Flags that influence the attach process. */
} fio_iodimm_attach_req2_t;

/** \brief Enumeration of potential results of an attach attempt. */
typedef enum 
{
    FIO_IODIMM_ATTACH_SUCCESS = 0,                     /**< Successfully attached */
    FIO_IODIMM_ATTACH_DEVICE_NOT_FOUND = 1,            /**< Could not find the device indicated by the selector. */
    FIO_IODIMM_ATTACH_IMPROPER_VERSION = 2,            /**< Attach failed due to an on-disk internal version mismatch */
    FIO_IODIMM_ATTACH_NO_EBMAP = 3,                    /**< Attach failed due to the ioDimm not having an LEB Map. */
    FIO_IODIMM_ATTACH_FAILED = 4,                      /**< Internal failure occurred while attempting to attach. */
    FIO_IODIMM_ATTACH_FAILED_PERM = 5,                 /**< User does not have permission to attach device. */
    FIO_IODIMM_ATTACH_FAILED_ALREADY_ATTACHED = 6,     /**< Device is already attached. */
    FIO_IODIMM_ATTACH_FAILED_NOT_IMPLEMENTED = 7,      /**< Reserved for internal use. */
    FIO_IODIMM_ATTACH_PCI_UNSUPPORTED = 8,             /**< Attaching using a PCI based selector is not supported.*/
    FIO_IODIMM_ATTACH_INVALID_INPUT = 9,               /**< Invalid input parameters*/
    FIO_IODIMM_ATTACH_IS_DIRTY = 10,                   /**< Attach failed because 'CLEAN_ONLY' set, but full scan is required. */ 
} fio_iodimm_attach_status_t;

/******************************************
 *             Detach Types               *
 ******************************************/
/** \brief Detach method. */
typedef enum
{
    FIO_IODIMM_DETACH_NORMAL = 0xA,     /**< Normal detach, may take a few seconds. */
    FIO_IODIMM_DETACH_IMMEDIATE = 0xB,  /**< Emergency immediate detach, may result in data loss and full rescan next
                                           attach attempt! */
} fio_iodimm_detach_method_t;

/** \brief Structure used to request an ioDimm to detach */
typedef struct fio_iodimm_detach_req2
{
    uint32_t                    lib_version;    /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    fio_iodimm_detach_method_t  detach_method;
    uint8_t                     force_detach;
} fio_iodimm_detach_req2_t;

/** \brief Results of a detach attempt. */
typedef enum {
    FIO_IODIMM_DETACH_SUCCESS = 0,                  /**< Successfully detached */
    FIO_IODIMM_DETACH_METHOD_NOT_SPECIFIED = 1,     /**< User did not explicitly specify a detach method */
    FIO_IODIMM_DETACH_DEVICE_NOT_FOUND = 2,         /**< Could not find the device pointed to by the ioDimm selector. */
    FIO_IODIMM_DETACH_FAILED = 3,                   /**< Internal failure occurred while attempting to detach. */
    FIO_IODIMM_DETACH_FAILED_DEVICE_BUSY = 4,       /**< Device was in use by the operating system and was not
                                                       ready to detach. */
    FIO_IODIMM_DETACH_FAILED_PERM = 5,              /**< User does not have permission to detach the device */
    FIO_IODIMM_DETACH_FAILED_ALREADY_DETACHED = 6,  /**< The device is already detached. */
    FIO_IODIMM_DETACH_FAILED_NOT_IMPLEMENTED = 7,   /**< Reserved for internal use. */
    FIO_IODIMM_DETACH_PCI_UNSUPPORTED = 8,          /**< Detaching using a PCI based selector is not supported.*/
    FIO_IODIMM_DETACH_INVALID_INPUT = 9             /**< Invalid input parameters*/
} fio_iodimm_detach_status_t;

/******************************************
 *             Update Types               *
 ******************************************/
/** \brief Structure used to request an ioDimm to update firmware */
typedef struct fio_iodimm_update_req2
{
    uint32_t                    lib_version;            /**< REQUIRED. Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    const char*                 fff_file_path;          /**< REQUIRED. Path to the firmware file to use during the update.
                                                           The path may be relative or absolute.*/
    char                        force;                  /**< OPTIONAL. Whether to force an update, even if firmware is out of date.
                                                           Set this to 0 for no force, 1 for force */
    char                        pretend;                /**< OPTIONAL. Whether to be in pretend mode, where the device is not actually
                                                           updated. If this value is set to 1, pretend mode is assumed. */
} fio_iodimm_update_req2_t;

/** \brief Results of a firmware update request. */

/*
 * There are two types of firmware.  
 *   - FPGA Firmware
 *   - SMP Firmware (System Management Processor)
 */
typedef enum {
    FIO_IODIMM_UPDATE_SUCCESS = 0,              /**< Successfully updated the device. A warm reboot is required */
    FIO_IODIMM_UPDATE_DEVICE_NOT_FOUND = 1,     /**< Could not find the device pointed to by the ioDimm selector. */
    FIO_IODIMM_UPDATE_FFF_FILE_NOT_FOUND = 2,   /**< Could not find the specified firmware file */
    FIO_IODIMM_UPDATE_FFF_PART_NOT_FOUND = 3,   /**< The FFF file is valid, but no firmware was found for the device's
                                                   part number */
    FIO_IODIMM_UPDATE_INVALID_FFF_FORMAT = 4,   /**< The FFF file is not in a valid format. */
    FIO_IODIMM_UPDATE_EEPROM_READ_FAIL = 5,     /**< Could not read the part number from the ioDimm */
    FIO_IODIMM_UPDATE_FIRMWARE_READ_FAIL = 6,   /**< Could not read the firmware version.  This is reported as an error
                                                   only if force is not enabled. */
    FIO_IODIMM_UPDATE_OLD_VERSION = 7,          /**< Selected FFF file was not loaded because it is an older
                                                   firmware version than what is currently loaded. */
    FIO_IODIMM_UPDATE_CUR_VERSION = 8,          /**< Selected FFF file was not loaded because it is the same
                                                   version as what is currently loaded. */
    FIO_IODIMM_UPDATE_DRIVER_INVALID_STATE = 9, /**< Driver is not in a valid state to allow a firmware upload
                                                   to occur. */
    FIO_IODIMM_UPDATE_FAILURE = 10,             /**< Internal failure occurred while updating */
    FIO_IODIMM_UPDATE_FAILURE_PERM = 11,        /**< User does not have permission to update the ioDimm */
    FIO_IODIMM_UPDATE_IOCTL_LOCK_FAILURE = 12,  /**< Device is locked for updating by a different process */
    FIO_IODIMM_UPDATE_SUCCESS_COLD_REBOOT = 13, /**< Device update success. A cold reboot is now required. */
    FIO_IODIMM_UPDATE_NOT_IMPLEMENTED = 14,     /**< Reserved for internal use. */
    FIO_IODIMM_UPDATE_SUCCESS_NO_REBOOT = 15,   /**< Device update success. No reboot is required. */
    FIO_IODIMM_UPDATE_SMP_NO_BOOTMODE = 16,     /**< Unable to put SMP into bootmode */
    FIO_IODIMM_UPDATE_SMP_NO_SMP = 17,          /**< Unable to locate SMP */
    FIO_IODIMM_UPDATE_SMP_FAILURE = 18,         /**< SMP update failure */
    FIO_IODIMM_UPDATE_INVALID_INPUT = 19,       /**< Invalid parameters. */

    FIO_IODIMM_UPDATE_UNSAFE            = 20,   /**< Upgrading from the current version to the proposed version is unsafe for user data. */

    FIO_IODIMM_UPDATE_ECC_MISMATCH = 21,        /**< Device and new firmware ECC lengths don't match. */
    FIO_IODIMM_UPDATE_ECC_ERROR = 22            /**< Failed to read ECC information. */
} fio_iodimm_update_status_t;

/** \brief Structure used to request a firmware version from a firmware file for a given ioDimm */
typedef struct fio_iodimm_fff_ver_req2
{
    uint32_t              lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    const char*           fff_file_path;  /**< Firmware file to get version from */
} fio_iodimm_fff_ver_req2_t;

/** \brief Enumeration of possible results when trying to get a firmware version for an ioDimm */
typedef enum {
    FIO_IODIMM_FFF_VER_SUCCESS = 0,             /**< Successfully got firmware version */
    FIO_IODIMM_FFF_VER_DEVICE_NOT_FOUND = 1,    /**< Device not found for given index */
    FIO_IODIMM_FFF_VER_FFF_FILE_NOT_FOUND = 2,  /**< Could not find the specified FFF file */
    FIO_IODIMM_FFF_VER_PART_NOT_FOUND = 3,      /**< The FFF file was valid, but no firmware was found for the device's
                                                   part number */
    FIO_IODIMM_FFF_VER_INVALID_FFF_FORMAT = 4,  /**< The FFF file is not in a valid format. */
    FIO_IODIMM_FFF_VER_READ_FAIL = 5,           /**< Could not read EEPROM information to get device's part number */
    FIO_IODIMM_FFF_VER_NOT_IMPLEMENTED = 6,     /**< Reserved for internal use. */
    FIO_IODIMM_FFF_VER_INVALID_INPUT = 7        /**< Invalid parameters. */
} fio_iodimm_fff_ver_status_t;

/******************************************
 *             Device Format Types        *
 ******************************************/
/** \brief Device format method to use */
typedef enum
{
    FIO_IODIMM_FORMAT_NORMAL        = 0xA,  /**< Perform a normal format, continuing on error until complete */
    FIO_IODIMM_FORMAT_QUIT_ON_ERR   = 0xB,  /**< Perform a format, but stop on the first error encountered. This should be
                                                used only for debugging purposes, as the device will be in an inconsistent
                                                state until the entire device has been formatted */
    FIO_IODIMM_FORMAT_NORM_PLUS_PPS = 0xE,  /**< Perform a normal format, and also create a Persistent Parameter Store on this ioDrive. */
} fio_iodimm_format_method_t;

/** \brief Format modes */
typedef enum
{
    FIO_IODIMM_FORMAT_USER_DEFINED_SIZE_MB  = 0xA,  /**< Format based on a fixed size in MiB. Use to also overformat or sparseformat by size. */
    FIO_IODIMM_FORMAT_USER_DEFINED_SIZE_PCT = 0xB,  /**< Format based on a percentage of the maximum format size */
    FIO_IODIMM_FORMAT_DEFAULT_SIZE          = 0xC,  /**< Format to the default size (100% of maximum format size) */
    FIO_IODIMM_FORMAT_SPARSE_SIZE_PCT       = 0xD,  /**< Sparse format to a percent of max sparse format size (max varies depending on driver version) */
} fio_iodimm_format_size_mode_t;

/** \brief Structure representing a request to low-level format an ioDimm */
typedef struct fio_iodimm_format_req2
{
    uint32_t                    lib_version;        /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_format_method_t  format_method;      /**< Format method */
    uint8_t                     force;              /**< Advanced use only, for specialized filesystems. This causes the
                                                        size checks to be overridden and allow the device to be
                                                        formatted to a larger size than it has available. Set to
                                                        non-zero to force*/
    uint64_t                    size_value;         /**< Size to format the device.  Allowable values depend on the mode selected.
                                                        If mode is percent then 1-100 is the range of valid values, where 100 is
                                                        the percent of max_format_size_mibytes, from fio_iodimm_get_format_sizes().
                                                        If you specify size in MiBytes, the valid range is from about 1 GiByte to
                                                        max_format_size_mibytes, and where 0 means 'default_format_size_mibytes'. */
    fio_iodimm_format_size_mode_t size_mode;        /**< Device size mode, percent or MiBytes. */
    uint32_t                    sector_size_bytes;  /**< Setting to non-zero will request that specific sector size. */
    uint32_t                    packet_head_bytes;  /**< Future use. Always set to 0. */
    int8_t                      fixed_percent_limit;/**< [out] If return status is FIO_IODIMM_FORMAT_FIXED_PCT_VIOLATED, this will
                                                        be set to the fixed percent the user must choose. */
    /* SDK API Version 6 and above */
    fio_format_type_t           format_target;      /**< [in] The type of the format: standard or special (ie directCache). */
    uint32_t                    pstore_min_entries; /**< [in] The minimum desired Perisistent Parameter Store capacity, in entries.
                                                        0 = no PPS will be created. 1 = default PPS entries, else actual number of entries. */
    uint32_t                    pstore_value_max_bytes; /**< [in] The maximum required PPS value size, in bytes. 0=use default. */
} fio_iodimm_format_req2_t;

typedef enum
{
    FIO_IODIMM_FORMAT_SUCCESS = 0,              /**< Successfully formatted the device */
    FIO_IODIMM_FORMAT_METHOD_NOT_SPECIFIED = 1, /**< Input error: the format method was not specified */
    FIO_IODIMM_FORMAT_INVALID_DRIVE_SIZE = 2,   /**< Input error: invalid drive size specified */
    FIO_IODIMM_FORMAT_FAIL_GOOD_D_LEB = 3,      /**< Flash has degraded, and the size request cannot be satisfied. */
    FIO_IODIMM_FORMAT_FAIL_GOOD_MD_LEB = 4,     /**< Obsolete: Flash has degraded and the device can no longer be formatted */
    FIO_IODIMM_FORMAT_FAIL_OPEN_CHANNEL = 5,    /**< Failed to open the channel. Make sure device is not in
                                                   minimal mode. */
    FIO_IODIMM_FORMAT_FAIL_LOCK_CHANNEL = 6,    /**< Failed to lock channel - another process may be doing a critical
                                                   task. Try again later. */
    FIO_IODIMM_FORMAT_QUIT_ON_ERR_BAD_LEB = 7,  /**< "Quit on error" was set, and an error was encountered */
    FIO_IODIMM_FORMAT_DEVICE_NOT_FOUND = 8,     /**< Could not find specified device */
    FIO_IODIMM_FORMAT_FAILURE = 9,              /**< Runtime formatting failure */
    FIO_IODIMM_FORMAT_FAILURE_NO_EBMAP = 10,    /**< Device is missing its EB map and cannot be formatted. Contact
                                                   customer support to address this issue */
    FIO_IODIMM_FORMAT_FAIL_NOMEM = 11,          /**< Not enough memory available to the driver to perform the format */
    FIO_IODIMM_FORMAT_FIXED_PCT_VIOLATED = 14,  /**< It is only possible to format this device to the default size and that
                                                   size was not specified in the format request. */
    FIO_IODIMM_FORMAT_NOT_IMPLEMENTED = 15,     /**< Reserved for internal use. */
    FIO_IODIMM_FORMAT_INVALID_SS_MIN = 16,      /**< Invalid minimum sector size and force was not used */
    FIO_IODIMM_FORMAT_INVALID_SS_MAX = 17,      /**< Invalid maximum sector size and force was not used */
    FIO_IODIMM_FORMAT_INVALID_SS_PWR2 = 18,     /**< Sector size must be power of 2 and force was not used */
    FIO_IODIMM_FORMAT_INVALID_SS_MULT = 19,     /**< Sector size does not match required multiplier */
    FIO_IODIMM_FORMAT_PCI_UNSUPPORTED = 20,     /**< Formatting using a PCI based selector is not supported.*/
    FIO_IODIMM_FORMAT_INVALID_INPUT = 21,       /**< Invalid input parameters */
    FIO_IODIMM_FORMAT_FAILED_PERM = 22,         /**< User does not have permission to format the device */
    FIO_IODIMM_FORMAT_NO_NV_DATA = 23,          /**< The NV data scan was skipped on driver load. Drive should not be formatted
                                                     or the warranty will be voided. */
} fio_iodimm_format_status_t;

/************************************************************************************
 *          Preallocation Types -- Allows ioDimm to be used as swap space           *
 ************************************************************************************/
/** \brief Structure used to request an ioDimm to start or stop preallocating 100% memory usage */
typedef struct fio_iodimm_prealloc_req
{
    uint32_t                lib_version;    /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint8_t                 enabled;        /**< If non-zero, ioDimm will begin preallocating worst case memory usage
                                                the next time the driver is loaded.  While preallocation is enabled,
                                                the device may be used as swap. */
} fio_iodimm_prealloc_req_t;

/** \brief Enumeration of potential results of a preallocation setting attempt. */
typedef enum {
    FIO_IODIMM_PREALLOC_SUCCESS = 0,                   /**< Successfully carried out request */
    FIO_IODIMM_PREALLOC_DEVICE_NOT_FOUND = 1,          /**< Could not find the device indicated by the selector. */
    FIO_IODIMM_PREALLOC_FAILED = 2,                    /**< Internal failure occurred while attempting to set preallocation state. */
    FIO_IODIMM_PREALLOC_FAILED_PERM = 3,               /**< User does not have permission to attach device. */
    FIO_IODIMM_PREALLOC_FAILED_NOT_IMPLEMENTED = 4,    /**< Reserved for internal use. */
    FIO_IODIMM_PREALLOC_FAILED_NOT_INITIALIZED = 5,    /**< fio_sdk_init must be called prior to this operation. */
    FIO_IODIMM_PREALLOC_INVALID_INPUT = 6              /**< Invalid input parameters */
} fio_iodimm_prealloc_status_t;

/** \brief Structure used to return information about how much memory is needed for preallocation. */
typedef struct fio_iodimm_prealloc_info
{
    uint32_t                lib_version;            /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint64_t                prealloc_memory_mib;    /**< The amount of memory required for preallocation, in MiB (1024*1024 bytes). */
} fio_iodimm_prealloc_info_t;


/************************************************************************************
 *          Write Regulation Types -- Control and monitor write regulation          *
 ************************************************************************************/
/** \brief Write regulation level values for the structure below. */
typedef enum fio_iodimm_write_regulation_level
{
    FIO_IODIMM_WRITE_REGULATION_NONE     = 0,
    FIO_IODIMM_WRITE_REGULATION_LIGHT    = 10,
    FIO_IODIMM_WRITE_REGULATION_MODERATE = 20,
    FIO_IODIMM_WRITE_REGULATION_HEAVY    = 30,
    FIO_IODIMM_WRITE_REGULATION_INVALID  = 255
} fio_iodimm_write_regulation_level_t;

/**
 * \brief Enumeration of the kinds of write regulation. This is used to retrieve data for
 *        the respective type. The TYPE_ACTUAL type is the regulation that actually gets
 *        applied depending on input from the other types.
 */
typedef enum fio_iodimm_write_regulation_type
{
    FIO_IODIMM_WRITE_REGULATION_TYPE_ACTUAL    = 0,
    FIO_IODIMM_WRITE_REGULATION_TYPE_LIFESPAN  = 1,
    FIO_IODIMM_WRITE_REGULATION_TYPE_POWER     = 2,
    FIO_IODIMM_WRITE_REGULATION_TYPE_THERMAL   = 3
} fio_iodimm_write_regulation_type_t;

/** \brief Structure used to return the lifetime extender state. */
typedef struct
{
    uint32_t lib_version;                    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t enabled;                         /**< bool: Non-zero if extender is enabled, 0 otherwise. */
    uint8_t capable;                         /**< bool: Non-zero if the product is capable of using the extender, 0 otherwise. */
    uint8_t allow_disable;                   /**< bool: Non-zero if extender may be disabled, 0 otherwise. */
    uint8_t allow_enable;                    /**< bool: Non-zero if extender may be re-enabled, 0 otherwise. */
    uint8_t allow_config;                    /**< bool: Non-zero if extender may be configured, 0 otherwise. */
    int64_t target_date;                     /**< User-configured target date for lifetime extender. */
} fio_iodimm_lifetime_extender_state_t;

/** \brief Structure used to enable and configure lifetime extender. */
typedef struct
{
    uint32_t lib_version;   /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint8_t  enable;        /**< boolean: set nonzero value to enable, 0 (zero) to disable. */
    int64_t target_date;    /**< Target date for governing (posix epoch format). Set to 0 for default*/
} fio_iodimm_lifetime_extender_enable_t;

/** \brief Structure used to return the write regulation level value for a given write regulation type. */
typedef struct fio_iodimm_write_regulation_level_info
{
    uint32_t lib_version;                       /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_write_regulation_type_t type;    /**< INPUT: set the type of data to request. */
    fio_iodimm_write_regulation_level_t level;  /**< The write regulation level value. */
} fio_iodimm_write_regulation_level_info_t;

/** \brief Enumeration of potential results of a lifetime extender enable/disable operation. */
typedef enum {
    FIO_IODIMM_EXTENDER_SUCCESS = 0,                   /**< Successfully carried out request */
    FIO_IODIMM_EXTENDER_DEVICE_NOT_FOUND = 1,          /**< Could not find the device indicated by the selector. */
    FIO_IODIMM_EXTENDER_FAILED = 2,                    /**< Internal failure occurred while attempting to enable the lifetime extender. */
    FIO_IODIMM_EXTENDER_FAILED_PERM = 3,               /**< User does not have permission to enable/disable extender. */
    FIO_IODIMM_EXTENDER_INVALID_INPUT = 4,             /**< Invalid input parameters */
    FIO_IODIMM_EXTENDER_PCI_UNSUPPORTED = 5,           /**< Attempting extender operations using a PCI based selector is not supported.*/
    FIO_IODIMM_EXTENDER_FAILED_ENABLE_DISALLOWED = 6,  /**< Attempt to enable was disallowed. */
    FIO_IODIMM_EXTENDER_FAILED_DISABLE_DISALLOWED = 7, /**< Attempt to disable was disallowed. */
    FIO_IODIMM_EXTENDER_FAILED_CONFIG_DISALLOWED = 8,  /**< Attempt to configure was disallowed. */
    FIO_IODIMM_EXTENDER_NOT_IMPLEMENTED = 9,           /**< Not implemented in this version of the SDK */
} fio_iodimm_extender_status_t;


/************************************************************************************
 *                               DEPRECATED STRUCTURES                              *
 ************************************************************************************/
#if FIO_SDK_DEPRECATED == 1 /* defined in version.h */

/**
 * \brief Select an ioDimm. This structure is used as the input to most operations. Depending on the selector mode,
 *  one of the members must be populated with the correct value.
 *
 *  This is required for those deprecated structures which have the fio_iodimm_selector embedded in them in order
 *  for them to preserve the correct size.
*/
typedef struct fio_iodimm_selector_pre_sdk3
{
    uint32_t                 lib_version;      /**< REQUIRED: Fusion SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_mode mode;             /**< REQUIRED: Specifies which of the remaining members will be populated */
    int                      index;            /**< CONDITIONAL: Device Index (ex: 0), populate if the selector mode is FIO_IODIMM_SELECT_INDEX */
    const char*              device_name;      /**< CONDITIONAL: Device Name (ex: /dev/fct0) , populate if the selector mode is FIO_IODIMM_SELECT_NAME */
    void*                    device_handle;    /**< INTERNAL: Internally used if device is opened via a non-PCI mode in open_iodimm_handle */
} fio_iodimm_selector_pre_sdk3_t;


/**
 * \brief structure used to register for callbacks from the SDK.  All operations will block until they
 *  are complete, so the only way to tell how far along they are is to register with callbacks via
 *  this structure
 *
 *  This is required for those deprecated structures which have the fio_progress_callback embedded in them in order
 *  for them to preserve the correct size.  Also, this version was not extensible since it does not have lib_version in it. */
typedef struct fio_progress_callback
{
    uint64_t        id;                 /**< Unique ID passed to the callback function so it can identify the purpose
                                           of the callback operation */
    fio_cb_fn_t     callback;           /**< Function to call with the progress updates */
    clock_t         callback_period_ms; /**< Number of milliseconds to wait between each callback */
    void*           context;            /**< Additional user context to propagate. This is passed to the callback
                                           function when it is invoked. */
} fio_progress_callback_t;



/** \brief Structure used to request an ioDimm to attach */
typedef struct fio_iodimm_attach_req
{
    uint32_t                            lib_version;    /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    fio_iodimm_selector_pre_sdk3_t      device;         /**< ioDimm selector pointing to the device to attempt to attach */
    fio_progress_callback_t*            callback;       /**< callback registration object if progress updates are desired.
                                                       May be NULL. */
} fio_iodimm_attach_req_t;

/** \brief Result of an attach request */
typedef struct fio_iodimm_attach_rsp
{
    fio_iodimm_attach_status_t status;
} fio_iodimm_attach_rsp_t;

/** \brief Structure used to request an ioDimm to detach */
typedef struct fio_iodimm_detach_req
{
    uint32_t                        lib_version;    /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    fio_iodimm_selector_pre_sdk3_t  device;
    fio_iodimm_detach_method_t      detach_method;
    fio_progress_callback_t*        callback;
    uint8_t                         force_detach;
} fio_iodimm_detach_req_t;

/** \brief Results of a detach request */
typedef struct fio_iodimm_detach_rsp
{
    fio_iodimm_detach_status_t  status;
} fio_iodimm_detach_rsp_t;

/** \brief Structure used to request an ioDimm to update firmware */
typedef struct fio_iodimm_update_req
{
    uint32_t                            lib_version;        /**< REQUIRED. Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_pre_sdk3_t      device;             /**< REQUIRED. ioDimm selector pointing to the device to attempt to update */
    const char*                         fff_file_path;      /**< REQUIRED. Path to the firmware file to use during the update.
                                                               The path may be relative or absolute.*/
    char                                force;              /**< OPTIONAL. Whether to force an update, even if firmware is out of date.
                                                               Set this to 0 for no force, 1 for force */
    fio_progress_callback_t*            callback;           /**< OPTIONAL. Callback reference. Set this to NULL if no callback is desired. */
    char                                pretend;            /**< OPTIONAL. Whether to be in pretend mode, where the device is not actually
                                                               updated. If this value is set to 1, pretend mode is assumed. */
} fio_iodimm_update_req_t;

/** \brief Result of an update request */
typedef struct fio_iodimm_update_rsp
{
    fio_iodimm_update_status_t status;          /**< The status of the the Update operation. */
} fio_iodimm_update_rsp_t;

/** \brief Structure for selecting a format mode and specifying the desired size based on that mode
 * This is required for the deprecated fio_iodimm_format_req
 */
typedef struct fio_iodimm_format_size_selector
{
    uint64_t                        size;   /**< Size to format the device.  Allowable values depend on the mode selected.
                                               If mode is percent then 1-100 is the range of valid values, where 100 is
                                                 the percent of max_format_size_mibytes, from fio_iodimm_get_format_sizes().
                                               If you specify size in MB (MiBytes), the valid range is 0 to
                                                 max_format_size_mibytes, and where 0 means 'default_format_size_mibytes'. */
    fio_iodimm_format_size_mode_t   mode;   /**< Device size mode, percent or MiBytes. */
} fio_iodimm_format_size_selector_t;

/** \brief Structure representing a request to low-level format an ioDimm */
typedef struct fio_iodimm_format_req
{
    uint32_t                          lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_pre_sdk3_t    device;         /**< ioDimm selector pointing to the device to format */
    fio_iodimm_format_method_t        format_method;  /**< Format method */
    fio_iodimm_format_size_selector_t size_selector;  /**< Size to format the device to */
    fio_progress_callback_t*          callback;       /**< Callback reference. Set this to NULL if no callback is desired. */
    uint8_t                           force;          /**< Advanced use only, for specialized filesystems. This causes the
                                                           size checks to be overridden and allow the device to be
                                                           formatted to a larger size than it has available. Set to
                                                           non-zero to force*/
    uint8_t                       make_new;           /**< Future use. Always set to 0. */
    uint32_t                      sector_size_bytes;  /**< Setting to non-zero will request that specific sector size. */
    uint32_t                      packet_head_bytes;  /**< Future use. Always set to 0. */
} fio_iodimm_format_req_t;

/** \brief Result of a format request */
typedef struct fio_iodimm_format_rsp
{
    fio_iodimm_format_status_t status;
    int8_t                     fixed_percent_limit;  /**< If status is FIO_IODIMM_FORMAT_FIXED_PCT_VIOLATED, this will
                                                        be set to the fixed percent the user must choose. */
} fio_iodimm_format_rsp_t;

/** \brief Structure used to request a firmware version from a firmware file for a given ioDimm */
typedef struct fio_iodimm_fff_ver_req
{
    uint32_t                        lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    fio_iodimm_selector_pre_sdk3_t  device;         /**< ioDimm selector pointing to the device to get the firmware version of */
    const char*                     fff_file_path;  /**< Firmware file to get version from */
} fio_iodimm_fff_ver_req_t;

/** \brief result of a firwmare version request */
typedef struct fio_iodimm_fff_ver_rsp
{
    fio_iodimm_fff_ver_status_t status;         /**< Result of the request. */
    fio_firmware_version_t      firmware_ver;   /**< Firmware version, if request was successful */
} fio_iodimm_fff_ver_rsp_t;

#endif /* FIO_SDK_DEPRECATED */

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_IODIMM_TYPES_H_ */
