/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_VOL_MGMT_TYPES_H_
#define _INCLUDE_FIO_PUBLIC_VOL_MGMT_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/common_types.h>
#define VOL_MGMT_MAX_FS_TYPE_LEN 16

typedef enum 
{
    FIO_VOLMAN_VT_UNKNOWN   = 0,    /*The volume type is unknown.*/
    FIO_VOLMAN_VT_SIMPLE    = 1,    /*The volume composed of partitions or extents from exactly one disk.*/
    FIO_VOLMAN_VT_SPAN      = 2,    /*The volume is composed of extents from more than one disk.*/
    FIO_VOLMAN_VT_STRIPE    = 3,    /*The volume is configured equivalent to RAID 0 */
    FIO_VOLMAN_VT_MIRROR    = 4,    /*The volume is configured equivalent to RAID 1.*/
    FIO_VOLMAN_VT_PARITY    = 5,    /*The volume is striped with parity, similar to RAID levels 3, 4, 5, and 6.*/
    FIO_VOLMAN_VT_MULTIPATH = 6,    /* Volume can be accessed via multiple paths for redundancy. */
    FIO_VOLMAN_VT_FAULTY    = 7,    /* The volume is, well, faulty. */
} fio_vol_mgmt_volume_type_t;


/**
 * \brief Represents a volume. A volume may have multiple mount points and may be shared across multiple drives.
 */
typedef struct fio_volume
{
    uint32_t                     lib_version;        /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    char*                        identifier;         /**< String that uniquely identifies the volume. This is OS-specific 
                                                          but is typically a GUID. */
    uint64_t                     mb_total;           /**< Size in MiB (1024^2) of total space on the volume. */
    uint64_t                     mb_used;            /**< Size in MiB (1024^2) of used space on the volume. NOTE: invalid if 0. */
    char*                        fs_type;            /**< Filesystem type being used by the volume ie NTFS, ext3, etc. */
    uint64_t                     mount_point_count;  /**< Number of mount points in mount_points. */
    char**                       mount_points;       /**< Array of names of mount points for this volume. */
    void*                        resource;           /**< Opaque OS-specific resource. Internal use only. Do not set or change. */
    /* SDK version 3 */
    char*                        volume_path;        /**< Path to the simple or RAID volume. For Windows, format is "Volume\{GUID}" */ 
    char*                        partition_path;     /**< Path or identifier for the base drive or partition, ie '/dev/fioa1'.
                                                          Windows format is &lt;Disk access path&gt;\&lt;Volume access path&gt;,
                                                          ie PhysicalDrive1\\HardDiskVolume3. */
    char*                        label;              /**< OS-specific volume label. Not available on all operating systems. */
    fio_vol_mgmt_volume_type_t   volume_type;        /**< The type of volume...simple or type of raid. */
} fio_volume_t;

typedef enum fio_storage_device_type
{
    FIO_STORAGE_DEVTYPE_UNKNOWN        = 0,         /**< The type of the device is unknown or has not yet been determined. */
    FIO_STORAGE_DEVTYPE_IOMEM_BLOCK    = 1,         /**< The device is an ioMemory device. */
    FIO_STORAGE_DEVTYPE_STD_BLOCK      = 2,         /**< The device is a standard block device, such as a hard drive. */
} fio_storage_device_type_t;

/**
 * \brief Represents a logical drive. A drive may be associated with multiple volumes.
 */
typedef struct fio_drive_handle
{
    uint32_t       lib_version;                     /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint32_t       volume_count;                    /**< Number of volumes this drive is associated with. */
    fio_volume_t** volumes;                         /**< Array of pointers to volumes. */
    void*          resource;                        /**< Opaque OS-specific resource. Internal use only. */
    /* SDK version 6 and above. */                  
    fio_storage_device_type_t type;                 /**< The type of this device: standard block, or an ioMemory block device. */
    char*          disk_name;                       /**< Name of the disk*/
} fio_drive_handle_t;

/**
 * \brief Represents a list of storage devices
 */
typedef struct fio_drive_handle_list
{
    uint32_t                lib_version;            /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    uint32_t                drive_handle_count;     /**< Number of storage devices in this list */             
    fio_drive_handle_t**    drive_handles;          /**< Array of storage devices */
} fio_drive_handle_list_t; 

#ifdef _MSC_VER
typedef enum
{
    MULTITHREADED     = 0x0,
    APARTMENTTHREADED = 0x2,
    DISABLE_OLE1DDE   = 0x4,
    SPEED_OVER_MEMORY = 0x8,
}fio_com_init_type_t;

typedef struct fio_vol_ini_com
{
    uint32_t              lib_version;           /**< Fusion SDK API version (populated by FIO_STRUCT_INIT) */
    fio_com_init_type_t   init_type;   
}fio_vol_ini_com_t;
#endif

typedef enum
{
    FIO_VOLMAN_SUCCESS              = 0,
    FIO_VOLMAN_OUT_OF_MEMORY        = 1,
    FIO_VOLMAN_INIT_FAILURE         = 2,
    FIO_VOLMAN_SERVICE_FAILURE      = 3,
    FIO_VOLMAN_DRIVE_NOT_FOUND      = 4,
    FIO_VOLMAN_VOLUME_NOT_FOUND     = 5,
    FIO_VOLMAN_INCONSISTENT_STATE   = 6,
    FIO_VOLMAN_GENERAL_FAILURE      = 7,
    FIO_VOLMAN_NOT_IMPLEMENTED      = 8,
    FIO_VOLMAN_NOT_ATTACHED         = 9,
    FIO_VOLMAN_OS_INFO_ERROR        = 10,   /**< Could not find OS-specific info where it was expected. */
    FIO_VOLMAN_INVALID_INPUT        = 11,   /**< Parameters to a function were incorrect or missing. */
    FIO_VOLMAN_END                  = 12,   /**< End of iterator items. No more items to retrieve. */
} fio_vol_mgmt_result_t;

#define FIO_VOLMAN_FST_UNKNOWN "Unknown"
#define FIO_VOLMAN_FST_RAW "WIN-RAW"
#define FIO_VOLMAN_FST_FAT "FAT"
#define FIO_VOLMAN_FST_FAT32 "FAT32"
#define FIO_VOLMAN_FST_NTFS "NTFS"
#define FIO_VOLMAN_FST_EXFAT "exFAT"
#define FIO_VOLMAN_FST_EXT "ext"
#define FIO_VOLMAN_FST_EXT2 "ext2"
#define FIO_VOLMAN_FST_EXT3 "ext3"
#define FIO_VOLMAN_FST_EXT4 "ext4"
#define FIO_VOLMAN_FST_XFS "XFS"
#define FIO_VOLMAN_FST_ZFS "ZFS"
#define FIO_VOLMAN_FST_NFS "nfs"
#define FIO_VOLMAN_FST_NFSD "nfsd"
#define FIO_VOLMAN_FST_TMPFS "tmpfs"
#define FIO_VOLMAN_FST_PROC "proc"
#define FIO_VOLMAN_FST_SYS "sysfs"
#define FIO_VOLMAN_FST_DEVPTS "devpts"
#define FIO_VOLMAN_FST_BINFMT_MISC "binfmt_misc"
#define FIO_VOLMAN_FST_RPC_PIPEFS "rpc_pipefs"
#define FIO_VOLMAN_FST_FUSECTL "fusectl"
#define FIO_VOLMAN_FST_FUSEBLK "fuseblk"

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_VOL_MGMT_TYPES_H_ */
