/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or 
 means without a valid, written license agreement with Fusion-io. Please 
 refer to the included "License" or "License.txt" file for terms and 
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <fio/public/iodimm_handle.h>
#include <fio/public/iodimm_operations.h>
#include <fio/public/common_types.h>

#define CALLBACK_INTERVAL_MS 1000;   /* Register for callback from SDK at 1 second interval */

#if defined(__SVR4) && defined(__sun) /* 32 bit for solaris */
#if !defined(PRIu64)
#define PRIu64 "llu"
#endif
#else  /* 64 bit platforms */
#if !defined(PRIu64)
#if !defined(FIO_IX32)
#define PRIu64 "lu"
#else
#define PRIu64 "llu"
#endif
#endif
#endif

void progress_callback(uint64_t id, int16_t percent_complete, void* context)
{
	printf("Attach operation ID %"PRIu64" percent complete: %d\n", id, percent_complete);
}

/**
 * \brief Sample application illustrating how to attach all of the ioDimm's found on the system.
 */
int main(int argc, char** argv)
{
    fio_status_result_t result;
    fio_iodimm_selector_t device;
    fio_iodimm_iterator_t iterator;
    fio_iodimm_attach_req2_t attach_request;
    fio_progress_callback2_t attach_callback;
    fio_iodimm_attach_status_t attach_result;
    
    /* Initialize the fio sdk.  Must always be done prior to using sdk functions */
    if (fio_sdk_init(FIO_SDK_API_VERSION) != 0)
    {
#ifdef WIN32
        printf("Failed to initialize SDK.  Is fio_abstraction.dll available in $PATH?\n");
#else
        printf("Failed to initialize SDK.  Is libfio_abstraction.so available in $PATH?\n");
#endif
        exit(-1);
    }

    /* initialize the device selector structure we will be using - SDK structures all require that
     * this be done prior to using them in an SDK call.  FIO_STATUS_INVALID_INPUT will be returned
     * from an SDK call that is given an uninitialized structure. */
    FIO_STRUCT_INIT(device, FIO_SDK_API_VERSION);
    FIO_STRUCT_INIT(iterator, FIO_SDK_API_VERSION);

    /* Initialize the an iodimm iterator and get the first device.  FIO_STATUS_SUCCESS is returned
     * if a device was found and put into the passed selector structure.  FIO_STATUS_DEVICE_NOT_FOUND
     * is returned when the end of the list is reached. */
    result = first_iodimm_handle(&iterator, &device);

    /* Loop while we are successful finding devices */
    while (result == FIO_STATUS_SUCCESS)
    {


        /* set up callback.  This is optional.  If this structure is not set up, no callbacks will occur. The attach
            call will block until the operation is complete, so a callback must be used if you want to know progress */
        /* Initialize the callback structure - this must be done for all SDK input and output structures prior to any SDK call. */
        FIO_STRUCT_INIT(attach_callback, FIO_SDK_API_VERSION);
        /* the function to call */
        attach_callback.callback = &progress_callback;
        /* how often to update with progress */
        attach_callback.callback_period_ms = CALLBACK_INTERVAL_MS;
        /* ID passed to callback function allowing identification of what operation the callback is coming from */
        attach_callback.id = device.index;
        /* We don't need to pass any extra context data for callback function to use, but if you needed to you
            could set it here. */
        attach_callback.context = 0;

        /* Initialize the request structure - this must be done prior to any SDK call.  Attach is currently
         * empty but it is reserved for future use (parameters could be added in the future).  It is valid to
         * pass NULL in instead, but this follows the pattern that all of the operations use.*/
        FIO_STRUCT_INIT(attach_request, FIO_SDK_API_VERSION);
        /* if there were parameters to set on the attach_request it would be done here */

        /* Attach the iodimm. */
        attach_result = fio_attach_iodimm2(&device, &attach_request, &attach_callback);

        /* Attach is done, check result */
        switch (attach_result)
        {
        case FIO_IODIMM_ATTACH_SUCCESS:
            printf("Successfully attached device (index %d)\n", device.index);
            break;
        /* Other cases not handled here for sake of code briefness.  Consult header for other possible return values */
        default:
            printf("Failed to attach device (index %d)\n", device.index);
            break;
        }

        /* Get the next device from the iterator */
        result = next_iodimm_handle(&iterator, &device);
    }

    return 0;
}
