/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or 
 means without a valid, written license agreement with Fusion-io. Please 
 refer to the included "License" or "License.txt" file for terms and 
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <fio/public/iodimm_handle.h>
#include <fio/public/iodimm_operations.h>
#include <fio/public/iodimm_mgmt.h>
#include <fio/public/common_types.h>

#if defined(__SVR4) && defined(__sun) /* 32 bit for solaris */
#if !defined(PRIu64)
#define PRIu64 "llu"
#endif
#else  /* 64 bit platforms */
#if !defined(PRIu64)
#if !defined(FIO_IX32)
#define PRIu64 "lu"
#else
#define PRIu64 "llu"
#endif
#endif
#endif

/**
 * \brief Sample application illustrating how to iterate over all of the ioDimm's on the system and
 *  print some of their information
 */
int main(int argc, char** argv)
{
    uint16_t i, num_iodimms;
    fio_status_result_t result;
    fio_iodimm_device_identifiers_t identifiers;
    fio_iodimm_product_info_t product_info;
    fio_iodimm_state_info_t state_info;
    /* Array of pointers to iodimm selectors - we will allocate later */
    fio_iodimm_selector_t** selectors = 0;
    
    /* Initialize the fio sdk.  Must always be done prior to using sdk functions */
    if (fio_sdk_init(FIO_SDK_API_VERSION) != 0)
    {
#ifdef WIN32
        printf("Failed to initialize SDK.  Is fio_abstraction.dll available in $PATH?\n");
#else
        printf("Failed to initialize SDK.  Is libfio_abstraction.so available in $PATH?\n");
#endif
        exit(-1);
    }

    /* First, determine all of the iodimms. Enumerate with size 0 to determine how much memory to allocate. 
        Alternatively, if you know which iodimm you want, you can manually populate the device selector prior to making
        the sdk request and skip using the enumeration API. */
    /* Note: check attach_iodimm.c for an example of using first_iodimm_handle and next_iodimm_handle in place of
     * fio_enumerate_iodimms. */
    num_iodimms = fio_enumerate_iodimms(selectors, 0);

    /* Now that we know how many iodimms there, allocate an array of pointers to hold the selectors.  This
        must be done outside of the SDK in order for the SDK to remain as stateless as possible. */
    selectors = (fio_iodimm_selector_t**)malloc(sizeof(fio_iodimm_selector_t*) * num_iodimms);

    /* Enumerate again with allocated memory.  This causes the array of pointers to be populated with values. The
     * selectors that are populated in this method have already been initialized with FIO_STRUCT_INIT. */
    fio_enumerate_iodimms(selectors, num_iodimms);

    /* Iterate over the devices */
    for (i = 0; i < num_iodimms; ++i)
    {
        /* Initialize the input structures - this must be done prior to any SDK call in order to guarantee 
            backwards compatibility */
        FIO_STRUCT_INIT(identifiers, FIO_SDK_API_VERSION);
        FIO_STRUCT_INIT(product_info, FIO_SDK_API_VERSION);
        FIO_STRUCT_INIT(state_info, FIO_SDK_API_VERSION);

        /* At this point, if you know you are going to be invoking multiple sdk functions, you should open the device
            handle.  If you do not open the device handle prior to calling an sdk function, that function will open it,
            use it, then close it.  Opening externally will allow all the sdk functions to use the same open handle.
            But if you are only going to be calling one sdk function, there is no benefit in opening the handle 
            externally.  Also, successfully opening the handle externally guarantees you won't get a device not found
            result when invoking sdk functions later. */
        result = open_iodimm_handle(selectors[i]);

        /* Only proceed in calling other sdk functions if open was successful.  See header for other return codes that
            may come from open_iodimm_handle, they are not shown here for sake of code briefness */
        if (result == FIO_STATUS_SUCCESS)
        {
            printf("Device information (index %d)\n", selectors[i]->index);

            /* Try to get the device identifiers */
            result = fio_iodimm_get_device_identifiers(selectors[i], &identifiers);
            
            if (result == FIO_STATUS_SUCCESS)
            {
                printf("Device Name: %s\n", identifiers.dev_name);
                printf("Block Device Name: %s\n", identifiers.blkdev_name);
                printf("Device Label: %s\n", identifiers.label);
            }

            /* Try to get the state info */
            result = fio_iodimm_get_device_state(selectors[i], &state_info);

            if (result == FIO_STATUS_SUCCESS)
            {
                printf("Device state: ");
                switch (state_info.state)
                {
                case FIO_IODIMM_STATE_DETACHED:
                    printf("Detached");
                    break;
                case FIO_IODIMM_STATE_ATTACHING:
                    printf("Attaching");
                    break;
                case FIO_IODIMM_STATE_SCANNING_METADATA:
                    printf("Scanning Metadata");
                    break;
                case FIO_IODIMM_STATE_SCANNING_DATA:
                    printf("Scanning Data");
                    break;
                case FIO_IODIMM_STATE_ATTACHED:
                    printf("Attached");
                    break;
                case FIO_IODIMM_STATE_DETACHING:
                    printf("Detaching");
                    break;
                case FIO_IODIMM_STATE_STOPPING:
                    printf("Stopping");
                    break;
                case FIO_IODIMM_STATE_STOPPED:
                    printf("Stopped");
                    break;
                case FIO_IODIMM_STATE_MINIMAL:
                    printf("Minimal Mode, Reason: ");
                    switch (state_info.minimal_mode_reason)
                    {
                    case FIO_MINMODE_REASON_UNKNOWN:
                        printf("Unknown");
                        break;
                    case FIO_MINMODE_REASON_FW_OUT_OF_DATE:
                        printf("Firmware out of date");
                        break;
                    case FIO_MINMODE_REASON_INSUFFICIENT_POWER:
                        printf("Insufficient power");
                        break;
                    case FIO_MINMODE_REASON_DUAL_PLANE_FAIL:
                        printf("dual plane mode could not be entered");
                        break;
                    case FIO_MINMODE_REASON_FORCED:
                        printf("User requested");
                        break;
                    case FIO_MINMODE_REASON_INTERNAL:
                        printf("Internal reason");
                        break;
                    case FIO_MINMODE_REASON_CARD_LIMIT_EXCEEDED:
                        printf("Max device limit for this type of device exceeded.");
                        break;
                    case FIO_MINMODE_REASON_UNSUPPORTED_OS:
                        printf("Unsupported OS");
                        break;
                    case FIO_MINMODE_REASON_INSUFFICIENT_MEMORY:
                        printf("Insufficient memory");
                        break;
                    case FIO_MINMODE_REASON_SMP_BOOTLOADER_MODE:
                        printf("SMP is in bootloader mode");
                        break;
                    case FIO_MINMODE_REASON_NOT_IN_MINMODE:
                        break;
                    }
                    break;
                case FIO_IODIMM_STATE_FAILED:
                    printf("Failed");
                    break;
                case FIO_IODIMM_STATE_UNKNOWN:
                    printf("Unknown");
                    break;
                }
                printf("\n");
            }

            /* Try to get the product info */
            result = fio_iodimm_get_product_info(selectors[i], &product_info);

            if (result == FIO_STATUS_SUCCESS)
            {
                printf("Product Info:\n");
                printf("\tProduct Name: %s\n", product_info.name);
                printf("\tProduct Number: %s\n", product_info.part_number_ext);
                printf("\tInternal Part Number: %s\n", product_info.part_number_int);
                printf("\tSerial Number: %05"PRIu64"\n", product_info.serial_number);
            }

            printf("\n\n");
        }

        /* Since we chose to open the iodimm handle externally, we must now close it */
        close_iodimm_handle(selectors[i]);
    }

    /* Free up the selectors allocated by fio_enumerate_iodimms */
    fio_free_enumerated_iodimms(selectors, num_iodimms);

    /* Free the array of pointers */
    free(selectors);

    return 0;
}
